# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Text processing actions for :class:`gaupol.Application`."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._


class ConfigureSpellCheckAction(gaupol.Action):

    """Set languages and spell-check targets."""

    def __init__(self):
        """Initialize a :class:`ConfigureSpellCheckAction` object."""
        gaupol.Action.__init__(self, "configure_spell_check")
        self.props.label = _("Co_nfigure Spell-check\342\200\246")
        self.props.tooltip = _("Set language and spell-check target")
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(aeidon.util.enchant_available())


class CheckSpellingAction(gaupol.Action):

    """Check for incorrect spelling."""

    def __init__(self):
        """Initialize a :class:`CheckSpellingAction` object."""
        gaupol.Action.__init__(self, "check_spelling")
        self.props.label = _("_Check Spelling")
        self.props.short_label = _("Spelling")
        self.props.stock_id = gtk.STOCK_SPELL_CHECK
        self.props.tooltip = _("Check for incorrect spelling")
        self.accelerator = "F7"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(aeidon.util.enchant_available())
        aeidon.util.affirm(gaupol.conf.spell_check.language)


class CorrectTextsAction(gaupol.Action):

    """Find and correct errors in texts."""

    def __init__(self):
        """Initialize a :class:`CorrectTextsAction` object."""
        gaupol.Action.__init__(self, "correct_texts")
        self.props.label = _("C_orrect Texts\342\200\246")
        self.props.tooltip = _("Find and correct errors in texts")
        self.accelerator = "O"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestEditorPage(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        self.dialog = gaupol.PreferencesDialog(self.application.window,
                                               self.application)

        self.page = self.dialog._editor_page
        self.dialog.show()

    def test__on_default_font_check_toggled(self):
        self.page._default_font_check.set_active(True)
        self.page._default_font_check.set_active(False)
        self.page._default_font_check.set_active(True)

    def test__on_font_button_font_set(self):
        self.page._default_font_check.set_active(False)
        self.page._font_button.set_font_name("Serif 12")
        self.page._font_button.emit("font-set")

    def test__on_length_cell_check_toggled(self):
        self.page._length_cell_check.set_active(True)
        self.page._length_cell_check.set_active(False)
        self.page._length_cell_check.set_active(True)

    def test__on_length_combo_changed(self):
        self.page._length_combo.set_active(0)
        self.page._length_combo.set_active(1)

    def test__on_length_edit_check_toggled(self):
        self.page._length_edit_check.set_active(True)
        self.page._length_edit_check.set_active(False)
        self.page._length_edit_check.set_active(True)

    def test__on_spell_check_check_toggled(self):
        self.page._spell_check_check.set_active(True)
        self.page._spell_check_check.set_active(False)
        self.page._spell_check_check.set_active(True)


class TestExtensionPage(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        self.dialog = gaupol.PreferencesDialog(self.application.window,
                                               self.application)

        self.page = self.dialog._extension_page
        self.dialog.show()

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test__on_about_button_clicked(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_CLOSE
        selection = self.page._tree_view.get_selection()
        selection.select_path(0)
        self.page._about_button.clicked()

    def test__on_about_dialog_url_clicked(self):
        self.page._on_about_dialog_url_clicked(None, gaupol.HOMEPAGE_URL)

    def test__on_tree_view_cell_toggled(self):
        column = self.page._tree_view.get_columns()[0]
        renderer = column.get_cell_renderers()[0]
        renderer.emit("toggled", 0)
        renderer.emit("toggled", 0)
        renderer.emit("toggled", 0)

    def test__on_tree_view_selection_changed(self):
        selection = self.page._tree_view.get_selection()
        selection.unselect_all()
        selection.select_path(0)
        column = self.page._tree_view.get_columns()[0]
        renderer = column.get_cell_renderers()[0]
        renderer.emit("toggled", 0)
        selection.unselect_all()
        selection.select_path(0)


class TestFilePage(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        self.dialog = gaupol.PreferencesDialog(self.application.window,
                                               self.application)

        self.page = self.dialog._file_page
        self.dialog.show()

    @aeidon.deco.monkey_patch(gaupol.util, "run_dialog")
    def test__on_add_button_clicked(self):
        def run_dialog(dialog):
            selection = dialog._tree_view.get_selection()
            selection.select_path(0)
            return gtk.RESPONSE_OK
        gaupol.util.run_dialog = run_dialog
        self.page._add_button.emit("clicked")

    def test__on_auto_check_toggled(self):
        self.page._auto_check.set_active(True)
        self.page._auto_check.set_active(False)
        self.page._auto_check.set_active(True)

    def test__on_down_button_clicked(self):
        selection = self.page._tree_view.get_selection()
        selection.select_path(0)
        self.page._down_button.emit("clicked")

    def test__on_locale_check_toggled(self):
        self.page._locale_check.set_active(True)
        self.page._locale_check.set_active(False)
        self.page._locale_check.set_active(True)

    def test__on_remove_button_clicked(self):
        selection = self.page._tree_view.get_selection()
        selection.select_path(0)
        self.page._remove_button.emit("clicked")

    def test__on_up_button_clicked(self):
        selection = self.page._tree_view.get_selection()
        selection.select_path(1)
        self.page._up_button.emit("clicked")


class TestPreviewPage(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        self.dialog = gaupol.PreferencesDialog(self.application.window,
                                               self.application)

        self.page = self.dialog._preview_page
        self.dialog.show()

    def test__init____use_built_in(self):
        gaupol.conf.preview.use_custom_command = False
        self.dialog = gaupol.PreferencesDialog(self.application.window,
                                               self.application)

    def test__init____use_custom(self):
        gaupol.conf.preview.use_custom_command = True
        self.dialog = gaupol.PreferencesDialog(self.application.window,
                                               self.application)

    def test__on_app_combo_changed(self):
        store = self.page._app_combo.get_model()
        for i in range(len(store) - 2):
            self.page._app_combo.set_active(i)
        self.page._app_combo.set_active(i + 2)

    def test__on_command_entry_changed(self):
        self.page._app_combo.set_active(0)
        self.page._command_entry.set_text("test")

    def test__on_force_utf_8_check_toggled(self):
        self.page._force_utf_8_check.set_active(True)
        self.page._force_utf_8_check.set_active(False)
        self.page._force_utf_8_check.set_active(True)

    def test__on_offset_spin_value_changed(self):
        self.page._offset_spin.set_value(13)
        self.page._offset_spin.set_value(-13)


class TestPreferencesDialog(gaupol.TestCase):

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        self.application = self.new_application()
        self.dialog = gaupol.PreferencesDialog(self.application.window,
                                               self.application)

        self.dialog.show()



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk

from .test_file import _TestFileDialog


class TestMultiSaveDialog(_TestFileDialog):

    def setup_method(self, method):
        self.application = self.new_application()
        self.application.add_page(self.new_page())
        self.application.add_page(self.new_page())
        self.dialog = gaupol.MultiSaveDialog(gtk.Window(), self.application)
        self.dialog.show()

    def test__on_response(self):
        self.dialog.response(gtk.RESPONSE_CANCEL)

    def test_get_format(self):
        for format in aeidon.formats:
            self.dialog.set_format(format)
            value = self.dialog.get_format()
            assert value == format

    def test_get_newline(self):
        for newline in aeidon.newlines:
            self.dialog.set_newline(newline)
            value = self.dialog.get_newline()
            assert value == newline

    def test_set_format(self):
        for format in aeidon.formats:
            self.dialog.set_format(format)
            value = self.dialog.get_format()
            assert value == format

    def test_set_newline(self):
        for newline in aeidon.newlines:
            self.dialog.set_newline(newline)
            value = self.dialog.get_newline()
            assert value == newline



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Searching for and replacing text."""

import aeidon
import gaupol


class SearchAgent(aeidon.Delegate):

    """Searching for and replacing text."""

    def __init__(self, master):
        """Initialize a :class:`SearchAgent` object."""
        aeidon.Delegate.__init__(self, master)
        self._search_dialog = None

    def _on_search_dialog_response(self, *args):
        """Hide the search dialog."""
        self._search_dialog.hide()

    @aeidon.deco.export
    def _on_find_and_replace_activate(self, *args):
        """Search for and replace text."""
        if self._search_dialog is not None:
            return self._search_dialog.present()
        self._search_dialog = gaupol.SearchDialog(self)
        aeidon.util.connect(self, "_search_dialog", "response")
        # Do not destroy the dialog, but rather hide based on response.
        self._search_dialog.connect("delete-event", lambda *args: True)
        self._search_dialog.show()

    @aeidon.deco.export
    def _on_find_next_activate(self, *args):
        """Search forwards for same text."""
        self._search_dialog.next()

    @aeidon.deco.export
    def _on_find_previous_activate(self, *args):
        """Search backwards for same text."""
        self._search_dialog.previous()



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Time and frame editing actions for :class:`gaupol.Application`."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._


class AdjustDurationsAction(gaupol.Action):

    """Lengthen or shorten durations."""

    def __init__(self):
        """Initialize an :class:`AdjustDurationsAction` object."""
        gaupol.Action.__init__(self, "adjust_durations")
        self.props.label = _("Adjust _Durations\342\200\246")
        self.props.tooltip = _("Lengthen or shorten durations")
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class ConvertFramerateAction(gaupol.Action):

    """Change positions for a different framerate."""

    def __init__(self):
        """Initialize a :class:`ConvertFramerateAction` object."""
        gaupol.Action.__init__(self, "convert_framerate")
        self.props.label = _("Convert _Framerate\342\200\246")
        self.props.tooltip = _("Change positions for a different framerate")
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.main_file is not None)


class PreviewAction(gaupol.Action):

    """Preview from selected position with a video player."""

    def __init__(self):
        """Initialize a :class:`PreviewAction` object."""
        gaupol.Action.__init__(self, "preview")
        self.props.label = _("_Preview")
        self.props.stock_id = gtk.STOCK_MEDIA_PLAY
        self.props.tooltip = _("Preview from selected position "
                               "with a video player")

        self.accelerator = "P"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(page.project.video_path is not None)
        if gaupol.conf.preview.use_custom_command:
            aeidon.util.affirm(gaupol.conf.preview.custom_command)


class ShiftPositionsAction(gaupol.Action):

    """Make subtitles appear earlier or later."""

    def __init__(self):
        """Initialize a :class:`ShiftPositionsAction` object."""
        gaupol.Action.__init__(self, "shift_positions")
        self.props.label = _("_Shift Positions\342\200\246")
        self.props.tooltip = _("Make subtitles appear earlier or later")
        self.accelerator = "H"
        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)


class SpeechRecognitionAction(gaupol.Action):

    """Generate subtitles by voice and speech recognition."""

    def __init__(self):
        """Initialize a :class:`SpeechRecognitionAction` object."""
        gaupol.Action.__init__(self, "speech_recognition")
        self.props.label = _("Sp_eech Recognition\342\200\246")
        self.props.tooltip = _("Generate subtitles by voice "
                               "and speech recognition")

        self.action_group = "main-safe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(gaupol.util.gst_available())
        aeidon.util.affirm(gaupol.util.pocketsphinx_available())


class TransformPositionsAction(gaupol.Action):

    """Change positions by linear two-point correction."""

    def __init__(self):
        """Initialize a :class:`TransformPositionsAction` object."""
        gaupol.Action.__init__(self, "transform_positions")
        self.props.label = _("_Transform Positions\342\200\246")
        self.props.tooltip = _("Change positions by linear "
                               "two-point correction")

        self.action_group = "main-unsafe"

    def _affirm_doable(self, application, page):
        """Raise :exc:`aeidon.AffirmationError` if action cannot be done."""
        aeidon.util.affirm(page is not None)
        aeidon.util.affirm(len(page.project.subtitles) > 1)


__all__ = tuple(filter(lambda x: x.endswith("Action"), dir()))



# Copyright (C) 2005-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Previewing subtitles with a video player."""

import aeidon
import os
import string
import subprocess


class PreviewAgent(aeidon.Delegate):

    """Previewing subtitles with a video player."""

    # pylint: disable=W0201

    __metaclass__ = aeidon.Contractual

    def __init__(self, master):
        """Initialize a :class:`aeidon.PreviewAgent` object."""
        aeidon.Delegate.__init__(self, master)
        aeidon.util.connect(self, self, "notify::main_file")

    def _get_subtitle_path_require(self, doc, encoding=None, temp=False):
        if encoding is not None:
            assert aeidon.encodings.is_valid_code(encoding)

    def _get_subtitle_path(self, doc, encoding=None, temp=False):
        """Return path to a file to preview, either real or temporary.

        Raise :exc:`IOError` if writing to temporary file fails.
        Raise :exc:`UnicodeError` if encoding temporary file fails.
        """
        sfile = self.get_file(doc)
        if sfile is None or encoding != sfile.encoding:
            return self.new_temp_file(doc)
        if doc == aeidon.documents.MAIN:
            if not self.main_changed and not temp:
                return self.main_file.path
        if doc == aeidon.documents.TRAN:
            if not self.tran_changed and not temp:
                return self.tran_file.path
        return self.new_temp_file(doc)

    def _on_notify_main_file(self, *args):
        """Try to find the video file path if unset."""
        if not self.video_path:
            self.find_video()

    @aeidon.deco.export
    def find_video(self, extensions=None):
        """Find and return the video file path based on main file's path.

        `extensions` should be a sequence of video filename extensions or
        ``None`` for defaults. The video file is searched for in the same
        directory as the subtitle file. The subtitle file's filename without
        extension is assumed to start with or match the video file's filename
        without extension.
        """
        if self.main_file is None: return None
        extensions = list(extensions or (".3ivx",
                                         ".asf",
                                         ".avi",
                                         ".divx",
                                         ".flv",
                                         ".m2v",
                                         ".mkv",
                                         ".mov",
                                         ".mp4",
                                         ".mpeg",
                                         ".mpg",
                                         ".ogm",
                                         ".ogv",
                                         ".qt",
                                         ".rm",
                                         ".rmvb",
                                         ".swf",
                                         ".vob",
                                         ".wmv",
            # Keep extensions that are used by other file types than videos at
            # the end of the list, so that if there are multiple matches, these
            # ambiguous extensions would not be the ones chosen.
                                         ".ogg",
                                         ".dat"))

        # Add upper-case versions of all extensions.
        extensions = aeidon.util.flatten([[x, x.upper()] for x in extensions])

        dirname = os.path.dirname(self.main_file.path)
        basename_sub = os.path.basename(self.main_file.path)
        rootname_sub = os.path.splitext(basename_sub)[0]
        for extension in extensions:
            for video_path in filter(lambda x: x.endswith(extension),
                                     os.listdir(dirname)):

                basename_video = os.path.basename(video_path)
                rootname_video = os.path.splitext(basename_video)[0]
                if not rootname_sub.startswith(rootname_video): continue
                self.video_path = os.path.join(dirname, basename_video)
                return self.video_path
        return None

    def preview_require(self, *args, **kwargs):
        assert self.video_path is not None

    def preview_ensure(self, value, *args, **kwargs):
        assert os.path.isfile(value[2])

    @aeidon.deco.export
    def preview(self,
                position,
                doc,
                command,
                offset,
                encoding=None,
                temp=False):

        """Start video player with `command` from `position`.

        `command` can have variables ``$MILLISECONDS``, ``$SECONDS``,
        ``$SUBFILE`` and ``$VIDEOFILE``. `offset` should be the amount of
        seconds before `position` to start, which can be used to take into
        account that video player's can usually seek only to keyframes, which
        exist maybe ten seconds or so apart. `encoding` can be specified if
        different from `doc` file encoding. Use ``True`` for `temp` to always
        use a temporary file for preview regardless of whether the file is
        changed or not.

        Raise :exc:`IOError` if writing to temporary file fails.
        Raise :exc:`UnicodeError` if encoding temporary file fails.
        Raise :exc:`aeidon.ProcessError` if unable to start process.

        Return a three tuple of :class:`subprocess.POpen` instance, command
        with variables expanded and process standard output and error path.
        """
        sub_path = self._get_subtitle_path(doc, encoding, temp=temp)
        output_path = aeidon.temp.create(".output")
        output_fd = aeidon.temp.get_handle(output_path)
        seconds = max(0, self.calc.to_seconds(position) - offset)
        command = string.Template(command).safe_substitute(
            MILLISECONDS=("%.0f" % (seconds * 1000)),
            SECONDS=("%.3f" % seconds),
            SUBFILE=aeidon.util.shell_quote(sub_path),
            VIDEOFILE=aeidon.util.shell_quote(self.video_path))

        process = aeidon.util.start_process(command,
                                            stderr=subprocess.STDOUT,
                                            stdout=output_fd)

        return process, command, output_path



# Copyright (C) 2011 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk
import os
import sys
import traceback


class TestSidePane(gaupol.TestCase):

    def setup_method(self, method):
        directory = os.path.abspath(os.path.dirname(__file__))
        directory = os.path.abspath(os.path.join(directory, ".."))
        sys.path.insert(0, directory)
        try: mobj = __import__("side-pane", {}, {}, [])
        except ImportError:
            return traceback.print_exc()
        finally: sys.path.pop(0)
        gaupol.conf.register_extension("side_pane",
                                       {"width": 200,
                                        "page": "",
                                        "visible": True})

        self.application = self.new_application()
        self.side_pane = mobj.SidePane(self.application)

    def test__on_header_close_button_clicked(self):
        side_vbox = self.side_pane._paned.get_child1()
        header_hbox = side_vbox.get_children()[0]
        button = header_hbox.get_children()[-1]
        button.emit("clicked")
        assert not side_vbox.props.visible

    def test__show_header_menu(self):
        self.side_pane.add_page(gtk.TextView(), "test_1", "Test 1")
        self.side_pane.add_page(gtk.TextView(), "test_2", "Test 2")
        self.side_pane.add_page(gtk.TextView(), "test_3", "Test 3")
        self.side_pane._show_header_menu(gtk.gdk.Event(gtk.gdk.BUTTON_PRESS))

    def test_add_page(self):
        text_view = gtk.TextView()
        self.side_pane.add_page(text_view, "test", "Test")
        page = self.side_pane.get_current_page()
        assert page is text_view

    def test_get_current_page(self):
        text_view = gtk.TextView()
        self.side_pane.add_page(text_view, "test", "Test")
        page = self.side_pane.get_current_page()
        assert page is text_view

    def test_hide(self):
        self.side_pane.show()
        self.side_pane.hide()
        side_vbox = self.side_pane._paned.get_child1()
        assert not side_vbox.props.visible

    def test_remove(self):
        text_view = gtk.TextView()
        self.side_pane.add_page(text_view, "test", "Test")
        self.side_pane.remove()

    def test_remove_page(self):
        text_view = gtk.TextView()
        self.side_pane.add_page(text_view, "test", "Test")
        page = self.side_pane.get_current_page()
        assert page is text_view
        self.side_pane.remove_page(text_view)
        page = self.side_pane.get_current_page()
        assert page is None

    def test_set_current_page(self):
        text_view_1 = gtk.TextView()
        text_view_2 = gtk.TextView()
        self.side_pane.add_page(text_view_1, "test_1", "Test 1")
        self.side_pane.add_page(text_view_2, "test_2", "Test 2")
        self.side_pane.set_current_page(text_view_1)
        assert self.side_pane._conf.page == "test_1"
        self.side_pane.set_current_page(text_view_2)
        assert self.side_pane._conf.page == "test_2"

    def test_show(self):
        self.side_pane.hide()
        self.side_pane.show()
        side_vbox = self.side_pane._paned.get_child1()
        assert side_vbox.props.visible


class TestSidePaneExtension(gaupol.TestCase):

    def setup_method(self, method):
        directory = os.path.abspath(os.path.dirname(__file__))
        directory = os.path.abspath(os.path.join(directory, ".."))
        sys.path.insert(0, directory)
        try: mobj = __import__("side-pane", {}, {}, [])
        except ImportError:
            return traceback.print_exc()
        finally: sys.path.pop(0)
        self.extension = mobj.SidePaneExtension()
        self.application = self.new_application()
        self.extension.setup(self.application)

    def teardown_method(self, method):
        self.extension.teardown(self.application)
        gaupol.TestCase.teardown_method(self, self.application)

    def test_setup(self):
        assert hasattr(self.application, "side_pane")

    def test_teardown(self):
        self.extension.teardown(self.application)
        assert not hasattr(self.application, "side_pane")
        self.extension.setup(self.application)



# Copyright (C) 2005-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""GTK+ user interface for the Gaupol subtitle editor.

:var COMBO_SEPARATOR: String used as a separator for :class:`gtk.ComboBox`

   :attr:`COMBO_SEPARATOR` can be inserted into a combo box's model, and
   :func:`gaupol.util.separate_combo` will recognize it.

:var BUG_REPORT_URL: Web page where to submit new bug reports
:var EXTENSIONS_URL: Web page listing third party extensions
:var HOMEPAGE_URL: Web page of the Gaupol project
:var PREVIEW_HELP_URL: Documentation on the preview function
:var REGEX_HELP_URL: Documentation on regular expressions
:var SPEECH_RECOGNITION_HELP_URL: Documentation on speech recognition
:var WIKI_URL: Wiki documentation

:var fields: Enumerations for subtitle field types
:var length_units: Enumerations for length unit types
:var targets: Enumerations for action target types
:var toolbar_styles: Enumerations for toolbar style types

:var field_actions: Dictionary mapping :attr:`gaupol.formats` to actions
:var framerate_actions: Dictionary mapping :attr:`aeidon.framerates` to actions
:var mode_actions: Dictionary mapping :attr:`aeidon.modes` to actions

:var conf: Instance of :class:`gaupol.ConfigurationStore` used
"""

import aeidon
import glib
import gobject
import gtk
import os

__version__ = "0.19.2"
COMBO_SEPARATOR = "<separator/>"

glib.threads_init()
gtk.rc_parse(os.path.join(aeidon.DATA_DIR, "ui", "gtkrc"))

from gaupol.urls import *
from gaupol import util
from gaupol.enums.fields import *
from gaupol.enums.styles import *
from gaupol.enums.targets import *
from gaupol.enums.units import *
from gaupol.enumuim import *
from gaupol.errors import *
from gaupol.attrdict import *
from gaupol.config import *
conf = ConfigurationStore()
from gaupol import ruler
from gaupol.meta import *
from gaupol.entries import *
from gaupol.renderers import *
from gaupol.output import *
from gaupol.view import *
from gaupol.page import *
from gaupol.extension import *
from gaupol.dialogs.builder import *
from gaupol.dialogs.file import *
from gaupol.dialogs.open import *
from gaupol.dialogs.save import *
from gaupol.dialogs.video import *
from gaupol.dialogs.append import *
from gaupol.dialogs.about import *
from gaupol.dialogs.debug import *
from gaupol.dialogs.duration import *
from gaupol.dialogs.encoding import *
from gaupol.dialogs.framerate import *
from gaupol.dialogs.header import *
from gaupol.dialogs.insert import *
from gaupol.dialogs.language import *
from gaupol.dialogs.message import *
from gaupol.dialogs.multiclose import *
from gaupol.dialogs.multisave import *
from gaupol.dialogs.preferences import *
from gaupol.dialogs.previewerr import *
from gaupol.dialogs.recognition import *
from gaupol.dialogs.search import *
from gaupol.dialogs.shift import *
from gaupol.dialogs.spellcheck import *
from gaupol.dialogs.split import *
from gaupol.dialogs.textedit import *
from gaupol.dialogs.transform import *
from gaupol.assistants import *
from gaupol.action import *
from gaupol import actions
from gaupol.extension import *
from gaupol.extensionman import *
from gaupol import agents
from gaupol.application import *
from gaupol import main
from gaupol.unittest import *



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestMarkup(aeidon.TestCase):

    text = ("All things weird are normal\n"
            "in this whore of cities.")

    def setup_method(self, method):
        self.markup = aeidon.Markup()

    def test_bolden(self):
        args = (self.markup.bolden, self.text)
        self.raises(NotImplementedError, *args)

    def test_clean(self):
        text = self.markup.clean(self.text)
        assert text == self.text

    def test_colorize(self):
        args = (self.markup.colorize, self.text, "ff00ff")
        self.raises(NotImplementedError, *args)

    def test_decode(self):
        text = self.markup.decode(self.text)
        assert text == self.text

    def test_encode__b(self):
        text = ("<b>All</b> things weird are normal\n"
                "in this whore of cities.")
        assert self.markup.encode(text) == self.text

    def test_encode__color(self):
        text = ('<color=#ffffff>All</color> things weird are normal\n'
                'in this whore of cities.')
        assert self.markup.encode(text) == self.text

    def test_encode__font(self):
        text = ('<font=Sans>All things weird are normal\n'
                'in this whore of cities.</font>')
        assert self.markup.encode(text) == self.text

    def test_encode__i(self):
        text = ("<i>All things weird are normal\n"
                "in this whore of cities.</i>")
        assert self.markup.encode(text) == self.text

    def test_encode__size(self):
        text = ('All things weird are normal\n'
                'in this whore of <size=12>cities</size>.')
        assert self.markup.encode(text) == self.text

    def test_encode__u(self):
        text = ("All things weird are normal\n"
                "in this whore of <u>cities</u>.")
        assert self.markup.encode(text) == self.text

    def test_fontify(self):
        args = (self.markup.fontify, self.text, "sans")
        self.raises(NotImplementedError, *args)

    def test_italic_tag(self):
        assert self.markup.italic_tag is None

    def test_italicize(self):
        args = (self.markup.italicize, self.text)
        self.raises(NotImplementedError, *args)

    def test_scale(self):
        args = (self.markup.scale, self.text, 12)
        self.raises(NotImplementedError, *args)

    def test_tag(self):
        assert self.markup.tag is None

    def test_underline(self):
        args = (self.markup.underline, self.text)
        self.raises(NotImplementedError, *args)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Enumerations for framerate types."""

import aeidon
_ = aeidon.i18n._

__all__ = ("framerates",)


class Framerate23976(aeidon.EnumerationItem):

    label = _("23.976 fps")
    mpsub = "23.98"
    value = 24 / 1.001


class Framerate24000(aeidon.EnumerationItem):

    label = _("24.000 fps")
    mpsub = "24.00"
    value = 24.0


class Framerate25000(aeidon.EnumerationItem):

    label = _("25.000 fps")
    mpsub = "25.00"
    value = 25.0


class Framerate29970(aeidon.EnumerationItem):

    label = _("29.970 fps")
    mpsub = "29.97"
    value = 30 / 1.001


framerates = aeidon.Enumeration()
framerates.FPS_23_976 = Framerate23976()
framerates.FPS_24_000 = Framerate24000()
framerates.FPS_25_000 = Framerate25000()
framerates.FPS_29_970 = Framerate29970()



# Copyright (C) 2007-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Assistant to guide through multiple text correction tasks."""

from __future__ import division

import aeidon
import gaupol
import glib
import gtk
import os
import pango
import sys
_ = aeidon.i18n._

__all__ = ("TextAssistant", "TextAssistantPage")


class TextAssistantPage(gtk.VBox):

    """Baseclass for pages of :class:`TextAssistant`.

   :ivar description: One-line description used in the introduction page
   :ivar handle: Unique unlocalized name for internal references
   :ivar page_title: Short string used as configuration page title
   :ivar page_type: A GTK+ assistant page type constant
   :ivar title: Short title used in the introduction page

    Of these attributes, :attr:`description`, :attr:`handle` and :attr:`title`
    are only required for pages of type :attr:`gtk.ASSISTANT_PAGE_CONTENT`.
    """

    def __init__(self, assistant):
        """Initialize a :class:`TextAssistantPage` object."""
        gtk.VBox.__init__(self)
        self.assistant = assistant
        self.description = None
        self.handle = None
        self.page_title = None
        self.page_type = None
        self.title = None
        self.set_border_width(12)


class BuilderPage(TextAssistantPage):

    """Baseclass for pages of :class:`TextAssistant` built with GtkBuilder."""

    _widgets = ()

    def __init__(self, assistant, basename):
        """Initialize a :class:`BuilderPage` object."""
        TextAssistantPage.__init__(self, assistant)
        ui_file_path = os.path.join(aeidon.DATA_DIR,
                                    "ui",
                                    "text-assistant",
                                    basename)

        self._builder = gtk.Builder()
        self._builder.set_translation_domain("gaupol")
        self._builder.add_from_file(ui_file_path)
        self._builder.connect_signals(self)
        self._set_attributes(self._widgets)
        self._builder.get_object("vbox").reparent(self)

    def _set_attributes(self, widgets):
        """Assign all names in `widgets` as attributes of `self`."""
        for name in widgets:
            widget = self._builder.get_object(name)
            setattr(self, "_%s" % name, widget)


class IntroductionPage(BuilderPage):

    """Page for listing all text correction tasks."""

    _widgets = ("all_radio",
                "current_radio",
                "main_radio",
                "selected_radio",
                "tran_radio",
                "tree_view")

    def __init__(self, assistant):
        """Initialize a :class:`IntroductionPage` object."""
        BuilderPage.__init__(self, assistant, "intro-page.ui")
        self.page_title = _("Select Tasks and Target")
        self.page_type = gtk.ASSISTANT_PAGE_INTRO
        self._init_tree_view()
        self._init_values()

    def _init_tree_view(self):
        """Initialize the tree view of tasks."""
        store = gtk.ListStore(object, bool, str)
        self._tree_view.set_model(store)
        selection = self._tree_view.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        renderer = gtk.CellRendererToggle()
        renderer.props.activatable = True
        renderer.props.xpad = 6
        renderer.connect("toggled", self._on_tree_view_cell_toggled)
        column = gtk.TreeViewColumn("", renderer, active=1)
        self._tree_view.append_column(column)
        renderer = gtk.CellRendererText()
        renderer.props.ellipsize = pango.ELLIPSIZE_END
        column = gtk.TreeViewColumn("", renderer, markup=2)
        self._tree_view.append_column(column)

    def _init_values(self):
        """Initialize default values for widgets."""
        target = gaupol.conf.text_assistant.target
        self._all_radio.set_active(target == gaupol.targets.ALL)
        self._current_radio.set_active(target == gaupol.targets.CURRENT)
        self._selected_radio.set_active(target == gaupol.targets.SELECTED)
        field = gaupol.conf.text_assistant.field
        self._main_radio.set_active(field == gaupol.fields.MAIN_TEXT)
        self._tran_radio.set_active(field == gaupol.fields.TRAN_TEXT)

    def _on_all_radio_toggled(self, *args):
        """Save the selected target."""
        gaupol.conf.text_assistant.target = self.get_target()

    def _on_current_radio_toggled(self, *args):
        """Save the selected target."""
        gaupol.conf.text_assistant.target = self.get_target()

    def _on_main_radio_toggled(self, *args):
        """Save the selected field."""
        gaupol.conf.text_assistant.field = self.get_field()

    def _on_selected_radio_toggled(self, *args):
        """Save the selected target."""
        gaupol.conf.text_assistant.target = self.get_target()

    def _on_tran_radio_toggled(self, *args):
        """Save the selected field."""
        gaupol.conf.text_assistant.field = self.get_field()

    def _on_tree_view_cell_toggled(self, renderer, row):
        """Toggle and save task check button value."""
        store = self._tree_view.get_model()
        store[row][1] = not store[row][1]
        store[row][0].props.visible = store[row][1]
        pages = [x.handle for x in self.get_selected_pages()]
        gaupol.conf.text_assistant.pages = pages

    def get_field(self):
        """Return the selected field."""
        if self._main_radio.get_active():
            return gaupol.fields.MAIN_TEXT
        if self._tran_radio.get_active():
            return gaupol.fields.TRAN_TEXT
        raise ValueError("Invalid field radio state")

    def get_selected_pages(self):
        """Return selected content pages."""
        store = self._tree_view.get_model()
        return [x[0] for x in store if x[1]]

    def get_target(self):
        """Return the selected target."""
        if self._selected_radio.get_active():
            return gaupol.targets.SELECTED
        if self._current_radio.get_active():
            return gaupol.targets.CURRENT
        if self._all_radio.get_active():
            return gaupol.targets.ALL
        raise ValueError("Invalid target radio state")

    def populate_tree_view(self, content_pages):
        """Populate the tree view with tasks from `content_pages`."""
        self._tree_view.get_model().clear()
        store = self._tree_view.get_model()
        pages = gaupol.conf.text_assistant.pages
        for page in content_pages:
            title = glib.markup_escape_text(page.title)
            description = glib.markup_escape_text(page.description)
            markup = "<b>%s</b>\n%s" % (title, description)
            page.props.visible = (page.handle in pages)
            store.append((page, page.handle in pages, markup))
        self._tree_view.get_selection().unselect_all()


class LocalePage(BuilderPage):

    """Page with script, language and coutry based pattern selection."""

    __metaclass__ = gaupol.ContractualGObject
    _ui_file_basename = NotImplementedError

    _widgets = ("country_combo",
                "country_label",
                "language_combo",
                "language_label",
                "script_combo",
                "script_label",
                "tree_view")

    def __init__(self, assistant):
        """Initialize a :class:`LocalePage` object."""
        BuilderPage.__init__(self, assistant, self._ui_file_basename)
        self.conf = None
        self._init_attributes()
        self._init_tree_view()
        self._init_combo_boxes()
        self._init_values()

    def _filter_patterns(self, patterns):
        """Return a subset of `patterns` to show."""
        return patterns

    def _get_country_ensure(self, value):
        if value is not None:
            assert aeidon.countries.is_valid(value)

    def _get_country(self):
        """Return the selected country or ``None``."""
        if not self._country_combo.props.sensitive: return None
        index = self._country_combo.get_active()
        if index < 0: return None
        store = self._country_combo.get_model()
        value = store[index][0]
        return (None if value == "other" else value)

    def _get_language_ensure(self, value):
        if value is not None:
            assert aeidon.languages.is_valid(value)

    def _get_language(self):
        """Return the selected language or ``None``."""
        if not self._language_combo.props.sensitive: return None
        index = self._language_combo.get_active()
        if index < 0: return None
        store = self._language_combo.get_model()
        value = store[index][0]
        return (None if value == "other" else value)

    def _get_script_ensure(self, value):
        if value is not None:
            assert aeidon.scripts.is_valid(value)

    def _get_script(self):
        """Return the selected script or ``None``."""
        if not self._script_combo.props.sensitive: return None
        index = self._script_combo.get_active()
        if index < 0: return None
        store = self._script_combo.get_model()
        value = store[index][0]
        return (None if value == "other" else value)

    def _init_attributes(self):
        """Initialize values of page attributes."""
        raise NotImplementedError

    def _init_combo(self, combo_box):
        """Initialize `combo_box` and populate with `items`."""
        store = gtk.ListStore(str, str)
        combo_box.set_model(store)
        renderer = gtk.CellRendererText()
        combo_box.pack_start(renderer, True)
        combo_box.add_attribute(renderer, "text", 1)
        combo_box.set_row_separator_func(gaupol.util.separate_combo)

    def _init_combo_boxes(self):
        """Initialize and populate combo boxes."""
        self._init_combo(self._script_combo)
        self._init_combo(self._language_combo)
        self._init_combo(self._country_combo)
        self._populate_script_combo()
        self._populate_language_combo()
        self._populate_country_combo()

    def _init_tree_view(self):
        """Initialize the tree view of individual corrections."""
        store = gtk.ListStore(object, bool, bool, str)
        store_filter = store.filter_new()
        store_filter.set_visible_column(1)
        self._tree_view.set_model(store_filter)
        selection = self._tree_view.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        renderer = gtk.CellRendererToggle()
        renderer.props.activatable = True
        renderer.props.xpad = 6
        renderer.connect("toggled", self._on_tree_view_cell_toggled)
        column = gtk.TreeViewColumn("", renderer, active=2)
        self._tree_view.append_column(column)
        renderer = gtk.CellRendererText()
        renderer.props.ellipsize = pango.ELLIPSIZE_END
        column = gtk.TreeViewColumn("", renderer, markup=3)
        self._tree_view.append_column(column)

    def _init_values(self):
        """Initialize default values for widgets."""
        pass

    def _on_country_combo_changed(self, combo_box):
        """Populate the tree view with a subset patterns."""
        self.conf.country = self._get_country() or ""
        self._populate_tree_view()

    def _on_language_combo_changed(self, combo_box):
        """Populate the tree view with a subset patterns."""
        language = self._get_language()
        sensitive = (language is not None)
        self._populate_country_combo()
        self._country_combo.set_sensitive(sensitive)
        self._country_label.set_sensitive(sensitive)
        self.conf.language = language or ""
        self._populate_tree_view()

    def _on_script_combo_changed(self, combo_box):
        """Populate the tree view with a subset patterns."""
        script = self._get_script()
        sensitive = (script is not None)
        self._populate_language_combo()
        self._language_combo.set_sensitive(sensitive)
        self._language_label.set_sensitive(sensitive)
        language = self._get_language()
        sensitive = (sensitive and (language is not None))
        self._populate_country_combo()
        self._country_combo.set_sensitive(sensitive)
        self._country_label.set_sensitive(sensitive)
        self.conf.script = script or ""
        self._populate_tree_view()

    def _on_tree_view_cell_toggled(self, renderer, path):
        """Toggle the check button value."""
        store_filter = self._tree_view.get_model()
        store = store_filter.get_model()
        row = store_filter.convert_path_to_child_path(path)[0]
        name = store[row][0].get_name(False)
        enabled = not store[row][2]
        for i in range(len(store)):
            # Toggle all patterns with the same name.
            if store[i][0].get_name(False) == name:
                store[i][0].enabled = enabled
                store[i][2] = enabled

    def _populate_combo(self, combo_box, items, active):
        """Populate `combo_box` with `items`."""
        store = combo_box.get_model()
        combo_box.set_model(None)
        store.clear()
        for code, name in items:
            store.append((code, name))
        if len(store) > 0:
            store.append((gaupol.COMBO_SEPARATOR, ""))
        store.append(("other", _("Other")))
        combo_box.set_active(len(store) - 1)
        for i in range(len(store)):
            if (store[i][0] == active) and active:
                combo_box.set_active(i)
        combo_box.set_model(store)

    def _populate_country_combo(self):
        """Populate the country combo box."""
        script = self._get_script()
        language = self._get_language()
        codes = self._manager.get_countries(script, language)
        names = map(aeidon.countries.code_to_name, codes)
        items = [(codes[i], names[i]) for i in range(len(codes))]
        items.sort(key=lambda x: x[1])
        self._populate_combo(self._country_combo, items, self.conf.country)

    def _populate_language_combo(self):
        """Populate the language combo box."""
        script = self._get_script()
        codes = self._manager.get_languages(script)
        names = map(aeidon.languages.code_to_name, codes)
        items = [(codes[i], names[i]) for i in range(len(codes))]
        items.sort(key=lambda x: x[1])
        self._populate_combo(self._language_combo, items, self.conf.language)

    def _populate_script_combo(self):
        """Populate the script combo box."""
        codes = self._manager.get_scripts()
        names = map(aeidon.scripts.code_to_name, codes)
        items = [(codes[i], names[i]) for i in range(len(codes))]
        items.sort(key=lambda x: x[1])
        self._populate_combo(self._script_combo, items, self.conf.script)

    def _populate_tree_view(self):
        """Populate the tree view with a subset patterns."""
        store_filter = self._tree_view.get_model()
        store = store_filter.get_model()
        store.clear()
        patterns = self._manager.get_patterns(self._get_script(),
                                              self._get_language(),
                                              self._get_country())

        patterns = self._filter_patterns(patterns)
        names_entered = set(())
        for pattern in patterns:
            name = pattern.get_name()
            visible = not (name in names_entered)
            names_entered.add(name)
            name = glib.markup_escape_text(name)
            description = pattern.get_description()
            description = glib.markup_escape_text(description)
            markup = "<b>%s</b>\n%s" % (name, description)
            store.append((pattern, visible, pattern.enabled, markup))
        self._tree_view.get_selection().unselect_all()

    def correct_texts(self, project, indices, doc):
        """Correct texts in `project`."""
        raise NotImplementedError


class CapitalizationPage(LocalePage):

    """Page for capitalizing texts in subtitles."""

    _ui_file_basename = "capitalization-page.ui"

    def _init_attributes(self):
        """Initialize values of page attributes."""
        self._manager = aeidon.PatternManager("capitalization")
        self.conf = gaupol.conf.capitalization
        self.description = _("Capitalize texts written in lower case")
        self.handle = "capitalization"
        self.page_title = _("Select Capitalization Patterns")
        self.page_type = gtk.ASSISTANT_PAGE_CONTENT
        self.title = _("Capitalize texts")

    def correct_texts(self, project, indices, doc):
        """Correct texts in `project`."""
        codes = (self._get_script(),
                 self._get_language(),
                 self._get_country())

        self._manager.save_config(*codes)
        patterns = self._manager.get_patterns(*codes)
        project.capitalize(indices, doc, patterns)


class CommonErrorPage(LocalePage):

    """Page for correcting common human and OCR errors."""

    _ui_file_basename = "common-error-page.ui"
    _widgets = ("human_check", "ocr_check") + LocalePage._widgets

    def _init_attributes(self):
        """Initialize values of page attributes."""
        self._manager = aeidon.PatternManager("common-error")
        self.conf = gaupol.conf.common_error
        self.description = _("Correct common errors made by humans "
                             "or image recognition software")

        self.handle = "common-error"
        self.page_title = _("Select Common Error Patterns")
        self.page_type = gtk.ASSISTANT_PAGE_CONTENT
        self.title = _("Correct common errors")

    def _filter_patterns(self, patterns):
        """Return a subset of `patterns` to show."""
        def use_pattern(pattern):
            classes = set(pattern.get_field_list("Classes"))
            return(bool(classes.intersection(set(self.conf.classes))))
        return filter(use_pattern, patterns)

    def _init_values(self):
        """Initialize default values for widgets."""
        self._human_check.set_active("Human" in self.conf.classes)
        self._ocr_check.set_active("OCR" in self.conf.classes)

    def _on_human_check_toggled(self, check_button):
        """Populate the tree view with a subset patterns."""
        if check_button.get_active():
            self.conf.classes.append("Human")
            self.conf.classes = sorted(set(self.conf.classes))
        elif "Human" in self.conf.classes:
            self.conf.classes.remove("Human")
        self._populate_tree_view()

    def _on_ocr_check_toggled(self, check_button):
        """Populate the tree view with a subset patterns."""
        if check_button.get_active():
            self.conf.classes.append("OCR")
            self.conf.classes = sorted(set(self.conf.classes))
        elif "OCR" in self.conf.classes:
            self.conf.classes.remove("OCR")
        self._populate_tree_view()

    def correct_texts(self, project, indices, doc):
        """Correct texts in `project`."""
        codes = (self._get_script(),
                 self._get_language(),
                 self._get_country())

        self._manager.save_config(*codes)
        patterns = self._manager.get_patterns(*codes)
        project.correct_common_errors(indices, doc, patterns)


class HearingImpairedPage(LocalePage):

    """Page for removing hearing impaired parts from subtitles."""

    _ui_file_basename = "hearing-impaired-page.ui"

    def _init_attributes(self):
        """Initialize values of page attributes."""
        self._manager = aeidon.PatternManager("hearing-impaired")
        self.conf = gaupol.conf.hearing_impaired
        self.description = _("Remove explanatory texts meant "
                             "for the hearing impaired")

        self.handle = "hearing-impaired"
        self.page_title = _("Select Hearing Impaired Patterns")
        self.page_type = gtk.ASSISTANT_PAGE_CONTENT
        self.title = _("Remove hearing impaired texts")

    def correct_texts(self, project, indices, doc):
        """Correct texts in `project`."""
        codes = (self._get_script(),
                 self._get_language(),
                 self._get_country())

        self._manager.save_config(*codes)
        patterns = self._manager.get_patterns(*codes)
        project.remove_hearing_impaired(indices, doc, patterns)


class JoinSplitWordsPage(BuilderPage):

    """Page for joining or splitting words based on spell-check suggestions."""

    __metaclass__ = gaupol.ContractualGObject
    _widgets = ("language_button", "join_check", "split_check")

    def __init___require(self, assistant):
        assert aeidon.util.enchant_available()

    def __init__(self, assistant):
        """Initialize a JoinSplitWordsPage object."""
        BuilderPage.__init__(self, assistant, "join-split-page.ui")
        self.description = _("Use spell-check suggestions to fix whitespace "
                             "detection errors of image recognition software")

        self.handle = "join-split-words"
        self.page_title = _("Set Options for Joining and Splitting Words")
        self.page_type = gtk.ASSISTANT_PAGE_CONTENT
        self.title = _("Join or Split Words")
        self._init_values()

    def _init_values(self):
        """Initialize default values for widgets."""
        language = gaupol.conf.spell_check.language
        try: label = aeidon.locales.code_to_name(language)
        except LookupError: label = self._language_button.get_label()
        self._set_language_button_label(label)
        self._join_check.set_active(gaupol.conf.join_split_words.join)
        self._split_check.set_active(gaupol.conf.join_split_words.split)

    def _on_join_check_toggled(self, check_button, *args):
        """Save value of join option."""
        gaupol.conf.join_split_words.join = check_button.get_active()

    def _on_language_button_clicked(self, button, *args):
        """Show a language dialog and update `button` label."""
        gaupol.util.set_cursor_busy(self.assistant)
        dialog = gaupol.LanguageDialog(self.assistant, False)
        gaupol.util.set_cursor_normal(self.assistant)
        gaupol.util.flash_dialog(dialog)
        language = gaupol.conf.spell_check.language
        try: label = aeidon.locales.code_to_name(language)
        except LookupError: label = self._language_button.get_label()
        self._set_language_button_label(label)

    def _on_split_check_toggled(self, check_button, *args):
        """Save value of split option."""
        gaupol.conf.join_split_words.split = check_button.get_active()

    def _set_language_button_label(self, text):
        """Set `text` as the language button label."""
        hbox = self._language_button.get_child()
        label = hbox.get_children()[0]
        label.set_text(text)

    def _show_error_dialog(self, message):
        """Show an error dialog after failing to load dictionary."""
        language = gaupol.conf.spell_check.language
        try: name = aeidon.locales.code_to_name(language)
        except LookupError: name = language
        title = _('Failed to load dictionary for language "%s"') % name
        dialog = gaupol.ErrorDialog(self.parent, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def correct_texts(self, project, indices, doc):
        """Correct texts in `project`."""
        import enchant
        language = gaupol.conf.spell_check.language
        if gaupol.conf.join_split_words.join:
            try: project.spell_check_join_words(indices, doc, language)
            except enchant.Error as message:
                return self._show_error_dialog(message)
        if gaupol.conf.join_split_words.split:
            try: project.spell_check_split_words(indices, doc, language)
            except enchant.Error as message:
                return self._show_error_dialog(message)


class LineBreakPage(LocalePage):

    """Page for breaking text into lines."""

    _ui_file_basename = "line-break-page.ui"

    def _init_attributes(self):
        """Initialize values of page attributes."""
        self._manager = aeidon.PatternManager("line-break")
        self.conf = gaupol.conf.line_break
        self.description = _("Break text into lines of defined length")
        self.handle = "line-break"
        self.page_title = _("Select Line-Break Patterns")
        self.page_type = gtk.ASSISTANT_PAGE_CONTENT
        self.title = _("Break lines")

    @property
    def _max_skip_length(self):
        """Return the maximum line length to skip."""
        if self.conf.use_skip_max_length:
            return self.conf.skip_max_length
        return sys.maxint

    @property
    def _max_skip_lines(self):
        """Return the maximum amount of lines to skip."""
        if self.conf.use_skip_max_lines:
            return self.conf.skip_max_lines
        return sys.maxint

    def correct_texts(self, project, indices, doc):
        """Correct texts in `project`."""
        codes = (self._get_script(),
                 self._get_language(),
                 self._get_country())

        self._manager.save_config(*codes)
        patterns = self._manager.get_patterns(*codes)
        length_func = gaupol.ruler.get_length_function(self.conf.length_unit)
        project.break_lines(indices=indices,
                            doc=doc,
                            patterns=patterns,
                            length_func=length_func,
                            max_length=self.conf.max_length,
                            max_lines=self.conf.max_lines,
                            max_deviation=self.conf.max_deviation,
                            skip=(self.conf.use_skip_max_length or
                                  self.conf.use_skip_max_lines),

                            max_skip_length=self._max_skip_length,
                            max_skip_lines=self._max_skip_lines)


class LineBreakOptionsPage(BuilderPage):

    """Page for editing line-break options."""

    _widgets = ("max_length_spin",
                "max_lines_spin",
                "max_skip_length_spin",
                "max_skip_lines_spin",
                "skip_length_check",
                "skip_lines_check",
                "skip_unit_combo",
                "unit_combo")

    def __init__(self, assistant):
        """Initialize a LineBreakOptionsPage object."""
        BuilderPage.__init__(self, assistant, "line-break-opts-page.ui")
        self.conf = gaupol.conf.line_break
        self.page_title = _("Set Line-Break Options")
        self.page_type = gtk.ASSISTANT_PAGE_CONTENT
        self._init_unit_combo(self._unit_combo)
        self._init_unit_combo(self._skip_unit_combo)
        self._init_values()

    def _init_unit_combo(self, combo_box):
        """Initialize line length unit `combo_box`."""
        store = gtk.ListStore(str)
        combo_box.set_model(store)
        for label in (x.label for x in gaupol.length_units):
            store.append((label,))
        renderer = gtk.CellRendererText()
        combo_box.pack_start(renderer, True)
        combo_box.add_attribute(renderer, "text", 0)

    def _init_values(self):
        """Initialize default values for widgets."""
        self._max_length_spin.set_value(self.conf.max_length)
        self._max_lines_spin.set_value(self.conf.max_lines)
        self._max_skip_length_spin.set_value(self.conf.skip_max_length)
        self._max_skip_lines_spin.set_value(self.conf.skip_max_lines)
        self._skip_length_check.set_active(self.conf.use_skip_max_length)
        self._skip_lines_check.set_active(self.conf.use_skip_max_lines)
        self._skip_unit_combo.set_active(self.conf.length_unit)
        self._unit_combo.set_active(self.conf.length_unit)

    def _on_max_length_spin_value_changed(self, spin_button):
        """Save maximum line length value."""
        self.conf.max_length = spin_button.get_value_as_int()

    def _on_max_lines_spin_value_changed(self, spin_button):
        """Save maximum line amount value."""
        self.conf.max_lines = spin_button.get_value_as_int()

    def _on_max_skip_length_spin_value_changed(self, spin_button):
        """Save maximum line length to skip value."""
        self.conf.skip_max_length = spin_button.get_value_as_int()

    def _on_max_skip_lines_spin_value_changed(self, spin_button):
        """Save maximum line amount to skip value."""
        self.conf.skip_max_lines = spin_button.get_value_as_int()

    def _on_skip_length_check_toggled(self, check_button):
        """Save skip by line length value."""
        use_skip = check_button.get_active()
        self.conf.use_skip_max_length = use_skip
        self._max_skip_length_spin.set_sensitive(use_skip)
        self._skip_unit_combo.set_sensitive(use_skip)

    def _on_skip_lines_check_toggled(self, check_button):
        """Save skip by line amount value."""
        use_skip = check_button.get_active()
        self.conf.use_skip_max_lines = use_skip
        self._max_skip_lines_spin.set_sensitive(use_skip)

    def _on_skip_unit_combo_changed(self, combo_box):
        """Save and sync length unit value of `combo_box."""
        index = combo_box.get_active()
        length_unit = gaupol.length_units[index]
        self.conf.length_unit = length_unit
        self._unit_combo.set_active(index)

    def _on_unit_combo_changed(self, combo_box):
        """Save and sync length unit value of `combo_box."""
        index = combo_box.get_active()
        length_unit = gaupol.length_units[index]
        self.conf.length_unit = length_unit
        self._skip_unit_combo.set_active(index)


class ProgressPage(BuilderPage):

    """Page for showing progress of text corrections."""

    _widgets = ("message_label",
                "progress_bar",
                "status_label",
                "task_label")

    def __init__(self, assistant):
        """Initialize a :class:`ProgressPage` object."""
        BuilderPage.__init__(self, assistant, "progress-page.ui")
        self._current_task = None
        self._total_tasks = None
        self.page_title = _("Correcting Texts")
        self.page_type = gtk.ASSISTANT_PAGE_PROGRESS
        self._init_values()

    def _init_values(self):
        """Initalize default values for widgets."""
        message = _("Each task is now being run on each project.")
        self._message_label.set_text(message)
        self.reset(100)

    def bump_progress(self, n_tasks=1):
        """Bump the current progress by `n_tasks`."""
        self.set_progress(self._current_task + n_tasks)

    def reset(self, total, clear_text=False):
        """Set `total` as the amount of tasks to be run."""
        self._current_task = 0
        self._total_tasks = total
        self.set_progress(0, total)
        self.set_project_name("")
        self.set_task_name("")
        if clear_text:
            self._progress_bar.set_text("")
        gaupol.util.iterate_main()

    def set_progress(self, current, total=None):
        """Set current as the task progress status."""
        total = total or self._total_tasks
        fraction = (current / total if total > 0 else 0)
        self._progress_bar.set_fraction(fraction)
        text = _("%(current)d of %(total)d tasks complete")
        self._progress_bar.set_text(text % locals())
        self._current_task = current
        self._total_tasks = total
        gaupol.util.iterate_main()

    def set_project_name(self, name):
        """Set `name` as the currently checked project."""
        text = _("Project: %s") % name
        text = glib.markup_escape_text(text)
        self._status_label.set_markup("<i>%s</i>" % text)
        gaupol.util.iterate_main()

    def set_task_name(self, name):
        """Set `name` as the currently performed task."""
        text = _("Task: %s") % name
        text = glib.markup_escape_text(text)
        self._task_label.set_markup("<i>%s</i>" % text)
        gaupol.util.iterate_main()


class ConfirmationPage(BuilderPage):

    """Page to confirm changes made after performing all tasks."""

    _widgets = ("mark_all_button",
                "preview_button",
                "remove_check",
                "tree_view",
                "unmark_all_button")

    def __init__(self, assistant):
        """Initialize a :class:`ConfirmationPage` object."""
        BuilderPage.__init__(self, assistant, "confirmation-page.ui")
        self.application = None
        self.conf = gaupol.conf.text_assistant
        self.doc = None
        self.page_title = _("Confirm Changes")
        self.page_type = gtk.ASSISTANT_PAGE_CONFIRM
        self._init_tree_view()
        self._init_values()

    def _add_text_column(self, index, title):
        """Add a multiline text column to the tree view."""
        renderer = gaupol.MultilineCellRenderer()
        renderer.set_show_lengths(True)
        renderer.props.editable = (index == 4)
        renderer.props.ellipsize = pango.ELLIPSIZE_END
        renderer.props.font = gaupol.util.get_font()
        renderer.props.yalign = 0
        column = gtk.TreeViewColumn(title, renderer, text=index)
        column.set_resizable(True)
        column.props.expand = True
        self._tree_view.append_column(column)

    def _can_preview(self):
        """Return ``True`` if preview is possible."""
        row = self._get_selected_row()
        if row is None: return False
        page = self._tree_view.get_model()[row][0]
        if page is None: return False
        return all((page.project.video_path, page.project.main_file))

    def _get_selected_row(self):
        """Return the selected row in the tree view or ``None``."""
        selection = self._tree_view.get_selection()
        store, itr = selection.get_selected()
        if itr is None: return None
        return store.get_path(itr)[0]

    def _init_tree_view(self):
        """Initialize the tree view of corrections."""
        # page, index, accept, original tex, new text
        store = gtk.ListStore(object, int, bool, str, str)
        self._tree_view.set_model(store)
        selection = self._tree_view.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        selection.connect("changed", self._on_tree_view_selection_changed)
        renderer = gtk.CellRendererToggle()
        renderer.props.activatable = True
        renderer.props.xpad = 6
        renderer.connect("toggled", self._on_tree_view_cell_toggled)
        column = gtk.TreeViewColumn(_("Accept"), renderer, active=2)
        column.set_resizable(True)
        self._tree_view.append_column(column)
        self._add_text_column(3, _("Original Text"))
        self._add_text_column(4, _("Corrected Text"))
        column = self._tree_view.get_column(2)
        renderer = column.get_cell_renderers()[0]
        renderer.connect("edited", self._on_tree_view_cell_edited)

    def _init_values(self):
        """Initialize default values for widgets."""
        self._remove_check.set_active(self.conf.remove_blank)
        self._preview_button.set_sensitive(False)

    def _on_mark_all_button_clicked(self, *args):
        """Set all accept column values to ``True``."""
        store = self._tree_view.get_model()
        for i in range(len(store)):
            store[i][2] = True

    def _on_preview_button_clicked(self, *args):
        """Preview original text in a video player."""
        row = self._get_selected_row()
        page = self._tree_view.get_model()[row][0]
        index = self._tree_view.get_model()[row][1]
        position = page.project.subtitles[index].start
        self.application.preview(page, position, self.doc)

    def _on_remove_check_toggled(self, check_button):
        """Save remove blank subtitles value."""
        self.conf.remove_blank = check_button.get_active()

    def _on_tree_view_cell_edited(self, renderer, row, text):
        """Edit text in the corrected text column."""
        store = self._tree_view.get_model()
        store[row][4] = unicode(text)

    def _on_tree_view_cell_toggled(self, renderer, row):
        """Toggle accept column value."""
        store = self._tree_view.get_model()
        store[row][2] = not store[row][2]

    def _on_tree_view_selection_changed(self, *args):
        """Update preview button sensitivity."""
        self._preview_button.set_sensitive(self._can_preview())

    def _on_unmark_all_button_clicked(self, *args):
        """Set all accept column values to ``False``."""
        store = self._tree_view.get_model()
        for i in range(len(store)):
            store[i][2] = False

    def get_confirmed_changes(self):
        """Return a sequence of changes marked as accepted."""
        changes = []
        store = self._tree_view.get_model()
        for row in (x for x in store if x[2]):
            page, index, accept, orig, new = row
            changes.append((page, index, orig, new))
        return tuple(changes)

    def populate_tree_view(self, changes):
        """Populate the tree view of changes to texts."""
        self._tree_view.get_model().clear()
        store = self._tree_view.get_model()
        for page, index, orig, new in changes:
            store.append((page, index, True, orig, new))
        self._tree_view.get_selection().unselect_all()


class TextAssistant(gtk.Assistant):

    """Assistant to guide through multiple text correction tasks."""

    def __init__(self, parent, application):
        """Initialize a TextAssistant object."""
        gtk.Assistant.__init__(self)
        self._confirmation_page = ConfirmationPage(self)
        self._introduction_page = IntroductionPage(self)
        self._previous_page = None
        self._progress_page = ProgressPage(self)
        self.application = application
        self._init_properties()
        self._init_signal_handlers()
        self.resize(*gaupol.conf.text_assistant.size)
        if gaupol.conf.text_assistant.maximized:
            self.maximize()
        self.set_modal(True)
        self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.set_transient_for(parent)

    def _copy_project(self, project):
        """Return a copy of `project` with some same properties."""
        copy = aeidon.Project(project.framerate)
        copy.main_file = project.main_file
        copy.tran_file = project.tran_file
        copy.subtitles = [x.copy() for x in project.subtitles]
        return copy

    def _correct_texts(self, assistant_pages):
        """Correct texts by all pages and present changes."""
        changes = []
        target = self._introduction_page.get_target()
        field = self._introduction_page.get_field()
        doc = gaupol.util.text_field_to_document(field)
        rows = self.application.get_target_rows(target)
        application_pages = self.application.get_target_pages(target)
        total = len(application_pages) * len(assistant_pages)
        self._progress_page.reset(total)
        for application_page in application_pages:
            name = application_page.get_main_basename()
            self._progress_page.set_project_name(name)
            project = application_page.project
            # Initialize a dummy project to apply corrections in
            # to be able to present those corrections for approval and
            # to finally be able to apply only approved corrections.
            dummy = self._copy_project(project)
            static_subtitles = dummy.subtitles[:]
            for page in assistant_pages:
                self._progress_page.set_task_name(page.title)
                page.correct_texts(dummy, rows, doc)
                self._progress_page.bump_progress()
            for i in range(len(static_subtitles)):
                orig = project.subtitles[i].get_text(doc)
                new = static_subtitles[i].get_text(doc)
                if orig == new: continue
                changes.append((application_page, i, orig, new))
        self._prepare_confirmation_page(doc, changes)
        self.set_current_page(self.get_current_page() + 1)

    def _init_properties(self):
        """Initialize assistant properties."""
        self.set_border_width(12)
        self.set_title(_("Correct Texts"))
        self.add_page(self._introduction_page)
        self.add_page(HearingImpairedPage(self))
        if aeidon.util.enchant_available():
            self.add_page(JoinSplitWordsPage(self))
        self.add_page(CommonErrorPage(self))
        self.add_page(CapitalizationPage(self))
        self.application.emit("text-assistant-request-pages", self)
        self.add_pages((LineBreakPage(self), LineBreakOptionsPage(self)))
        self.add_page(self._progress_page)
        self.add_page(self._confirmation_page)

    def _init_signal_handlers(self):
        """Initialize signal handlers."""
        aeidon.util.connect(self, self, "apply")
        aeidon.util.connect(self, self, "cancel")
        aeidon.util.connect(self, self, "close")
        aeidon.util.connect(self, self, "prepare")
        aeidon.util.connect(self, self, "window-state-event")

    def _on_apply(self, *args):
        """Apply accepted changes to projects."""
        gaupol.util.set_cursor_busy(self)
        edits = removals = 0
        changes = self._confirmation_page.get_confirmed_changes()
        target = self._introduction_page.get_target()
        application_pages = self.application.get_target_pages(target)
        field = self._introduction_page.get_field()
        doc = gaupol.util.text_field_to_document(field)
        description = _("Correcting texts")
        register = aeidon.registers.DO
        for page in application_pages:
            indices = [x[1] for x in changes if x[0] is page]
            texts = [x[3] for x in changes if x[0] is page]
            if indices and texts:
                page.project.replace_texts(indices, doc, texts)
                page.project.set_action_description(register, description)
                edits += len(indices)
            indices = [x for i, x in enumerate(indices) if not texts[i]]
            if indices and gaupol.conf.text_assistant.remove_blank:
                page.project.remove_subtitles(indices)
                page.project.group_actions(register, 2, description)
                removals += len(indices)
            page.view.columns_autosize()
        edits = edits - removals
        message = _("Edited %(edits)d and removed %(removals)d subtitles")
        self.application.flash_message(message % locals())
        gaupol.util.set_cursor_normal(self)

    def _on_cancel(self, *args):
        """Destroy assistant."""
        self._save_window_geometry()
        self.destroy()

    def _on_close(self, *args):
        """Destroy assistant."""
        self._save_window_geometry()
        self.destroy()

    def _on_prepare(self, assistant, page):
        """Prepare `page` to be shown next."""
        previous_page = self._previous_page
        self._previous_page = page
        if page is self._introduction_page:
            return self._prepare_introduction_page()
        pages = self._introduction_page.get_selected_pages()
        if page is self._progress_page:
            if previous_page is not self._confirmation_page:
                return self._prepare_progress_page(pages)

    def _on_window_state_event(self, window, event):
        """Save window maximization."""
        state = event.new_window_state
        maximized = bool(state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
        gaupol.conf.text_assistant.maximized = maximized

    def _prepare_confirmation_page(self, doc, changes):
        """Present `changes` and activate confirmation page."""
        count = len(changes)
        title = aeidon.i18n.ngettext("Confirm %d Change",
                                     "Confirm %d Changes", count) % count

        self.set_page_title(self._confirmation_page, title)
        self._confirmation_page.application = self.application
        self._confirmation_page.doc = doc
        self._confirmation_page.populate_tree_view(changes)
        self.set_page_complete(self._progress_page, True)

    def _prepare_introduction_page(self):
        """Prepare introduction page content."""
        n = self.get_n_pages()
        pages = map(self.get_nth_page, range(n))
        pages.remove(self._introduction_page)
        pages.remove(self._progress_page)
        pages.remove(self._confirmation_page)
        pages = filter(lambda x: hasattr(x, "correct_texts"), pages)
        self._introduction_page.populate_tree_view(pages)

    def _prepare_progress_page(self, pages):
        """Prepare progress page for `pages`."""
        self._progress_page.reset(0, True)
        self.set_page_complete(self._progress_page, False)
        gaupol.util.delay_add(10, self._correct_texts, pages)

    def _save_window_geometry(self):
        """Save the geometry of the assistant window."""
        if not gaupol.conf.text_assistant.maximized:
            gaupol.conf.text_assistant.size = list(self.get_size())

    def add_page(self, page):
        """Add `page` and configure its properties."""
        page.show_all()
        self.append_page(page)
        self.set_page_type(page, page.page_type)
        self.set_page_title(page, page.page_title)
        if page.page_type != gtk.ASSISTANT_PAGE_PROGRESS:
            self.set_page_complete(page, True)

    def add_pages(self, pages):
        """Add associated `pages` and configure their properties.

        The first one of `pages` must have a "correct_texts" attribute.
        The visibilities of other pages are kept in sync with the first page.
        """
        map(self.add_page, pages)
        def on_notify_visible(page, prop, pages):
            for page in pages[1:]:
                page.props.visible = pages[0].props.visible
        pages[0].connect("notify::visible", on_notify_visible, pages)



# Copyright (C) 2005-2008 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk


class _TestMessageDialog(gaupol.TestCase):

    def run__dialog(self):
        # pylint: disable=E1101
        self.dialog.run()
        self.dialog.destroy()


class TestErrorDialog(_TestMessageDialog):

    def setup_method(self, method):
        self.dialog = gaupol.ErrorDialog(gtk.Window(), "test", "test")
        self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.dialog.show()


class TestInfoDialog(_TestMessageDialog):

    def setup_method(self, method):
        self.dialog = gaupol.InfoDialog(gtk.Window(), "test", "test")
        self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.dialog.show()


class TestQuestionDialog(_TestMessageDialog):

    def setup_method(self, method):
        self.dialog = gaupol.QuestionDialog(gtk.Window(), "test", "test")
        self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.dialog.show()


class TestWarningDialog(_TestMessageDialog):

    def setup_method(self, method):
        self.dialog = gaupol.WarningDialog(gtk.Window(), "test", "test")
        self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.dialog.show()



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestTextAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    def test_break_lines(self):
        for subtitle in self.project.subtitles:
            subtitle.main_text = subtitle.main_text.replace(" ", "\n")
        manager = aeidon.PatternManager("common-error")
        self.project.break_lines(self.project.get_all_indices(),
                                 aeidon.documents.MAIN,
                                 manager.get_patterns("Latn"),
                                 length_func=len,
                                 max_length=44,
                                 max_lines=2,
                                 max_deviation=1)

        for subtitle in self.project.subtitles:
            assert subtitle.main_text.count("\n") <= 2

    def test_capitalize(self):
        for subtitle in self.project.subtitles:
            subtitle.main_text = "test. test i."
        manager = aeidon.PatternManager("capitalization")
        self.project.capitalize(self.project.get_all_indices(),
                                aeidon.documents.MAIN,
                                manager.get_patterns("Latn", "en"))

        for subtitle in self.project.subtitles:
            assert subtitle.main_text == "Test. Test I."

    def test_correct_common_errors(self):
        self.project.subtitles[0].main_text = "''Test''"
        self.project.subtitles[1].main_text = "123o456o789"
        manager = aeidon.PatternManager("common-error")
        self.project.correct_common_errors(self.project.get_all_indices(),
                                           aeidon.documents.MAIN,
                                           manager.get_patterns("Latn"))

        assert self.project.subtitles[0].main_text == '"Test"'

    def test_remove_hearing_impaired(self):
        orig_length = len(self.project.subtitles)
        self.project.subtitles[0].main_text = "[Boo] Test."
        self.project.subtitles[1].main_text = "[Boo]"
        manager = aeidon.PatternManager("hearing-impaired")
        patterns = manager.get_patterns("Latn")
        for pattern in patterns:
            pattern.enabled = True
        self.project.remove_hearing_impaired(self.project.get_all_indices(),
                                             aeidon.documents.MAIN,
                                             patterns)

        assert self.project.subtitles[0].main_text == "Test."
        assert len(self.project.subtitles) == orig_length - 1

    def test_spell_check_join_words(self):
        for subtitle in self.project.subtitles:
            subtitle.main_text = subtitle.main_text.replace("a", " a")
            subtitle.main_text = subtitle.main_text.replace("e", "e ")
        self.project.spell_check_join_words(self.project.get_all_indices(),
                                            aeidon.documents.MAIN,
                                            "en")

    def test_spell_check_join_words__enchant_error(self):
        import enchant
        self.raises(enchant.Error,
                    self.project.spell_check_join_words,
                    None, aeidon.documents.MAIN, "xx")

    def test_spell_check_split_words(self):
        for subtitle in self.project.subtitles:
            subtitle.main_text = subtitle.main_text.replace("s ", "s")
            subtitle.main_text = subtitle.main_text.replace("y ", "y")
        self.project.spell_check_split_words(self.project.get_all_indices(),
                                             aeidon.documents.MAIN,
                                             "en")

    def test_spell_check_split_words__enchant_error(self):
        import enchant
        self.raises(enchant.Error,
                    self.project.spell_check_split_words,
                    None, aeidon.documents.MAIN, "xx")



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Time and frame calculator."""

from __future__ import division

import aeidon

__all__ = ("Calculator",)


class Calculator(object):

    """Time and frame calculator.

    Times are handled as strings, frames as integers and seconds as floats.
    Only one instance of :class:`Calculator` exists for a given framerate.
    """

    _instances = {}

    def __init__(self, framerate=None):
        """Initialize a :class:`Calculator` object."""
        framerate = framerate or aeidon.framerates.FPS_23_976
        self._framerate = framerate.value

    def __new__(cls, framerate=None):
        """Return possibly existing instance for `framerate`."""
        framerate = framerate or aeidon.framerates.FPS_23_976
        if not framerate in cls._instances:
            cls._instances[framerate] = object.__new__(cls)
        return cls._instances[framerate]

    def add_seconds_to_time(self, time, seconds):
        """Add amount of `seconds` to `time`."""
        seconds = self.time_to_seconds(time) + seconds
        return self.seconds_to_time(seconds)

    def add_times(self, x, y):
        """Add time `y` to time `x`."""
        x = self.time_to_seconds(x)
        y = self.time_to_seconds(y)
        return self.seconds_to_time(x + y)

    def compare(self, x, y):
        """Return 1 if `x` is greater, 0 if equal and -1 if `y` greater."""
        if isinstance(x, basestring):
            return self.compare_times(x, y)
        if isinstance(x, int):
            return cmp(x, y)
        raise ValueError("Invalid type for x: %s" % repr(type(x)))

    def compare_times(self, x, y):
        """Return 1 if `x` is greater, 0 if equal and -1 if `y` greater."""
        negative = (x.startswith("-") and y.startswith("-"))
        return (negative * -2 + 1) * cmp(x, y)

    def frame_to_seconds(self, frame):
        """Convert `frame` to seconds."""
        return frame / self._framerate

    def frame_to_time(self, frame):
        """Convert `frame` to time."""
        seconds = self.frame_to_seconds(frame)
        return self.seconds_to_time(seconds)

    def get_frame_duration(self, x, y):
        """Return duration from frame `x` to frame `y`."""
        return y - x

    def get_middle(self, x, y):
        """Return time or frame halfway between `x` and `y`.

        >>> calc = aeidon.Calculator()
        >>> calc.get_middle(33, 77)
        55
        >>> calc.get_middle("00:00:00.123", "00:00:10,456")
        '00:00:05.289'
        """
        if isinstance(x, basestring):
            x = self.time_to_seconds(x)
            y = self.time_to_seconds(y)
            return self.seconds_to_time((x + y) / 2)
        if isinstance(x, int):
            return int(round((x + y) / 2, 0))
        raise ValueError("Invalid type for x: %s" % repr(type(x)))

    def get_time_duration(self, x, y):
        """Return duration from time `x` to time `y`."""
        x = self.time_to_seconds(x)
        y = self.time_to_seconds(y)
        return self.seconds_to_time(y - x)

    def is_valid_time(self, time):
        """Return ``True`` if `time` is a valid time string."""
        if time.startswith("-"):
            time = time[1:]
        try:
            hours = int(time[:2])
            minutes = int(time[3:5])
            seconds = int(time[6:8])
            mseconds = int(time[9:])
        except ValueError:
            return False
        return (0 <= hours <= 99 and
                0 <= minutes <= 59 and
                0 <= seconds <= 59 and
                0 <= mseconds <= 999)

    def parse_time(self, time):
        """Parse syntactically sloppy `time` to valid format.

        >>> calc = aeidon.Calculator()
        >>> calc.parse_time("1:2:3,4")
        '01:02:03.400'
        """
        time = time.strip()
        sign = ("-" if time.startswith("-") else "")
        time = time.replace("-", "")
        time = time.replace(",", ".")
        hours, minutes, seconds = map(float, time.split(":"))
        return "%s%02.0f:%02.0f:%02.0f.%03.0f" % (sign,
                                                  hours,
                                                  minutes,
                                                  int(seconds),
                                                  (seconds % 1) * 1000)

    def round_time(self, time, decimals):
        """Round `time` to amount of `decimals` in seconds."""
        seconds = self.time_to_seconds(time)
        seconds = round(seconds, decimals)
        return self.seconds_to_time(seconds)

    def seconds_to_frame(self, seconds):
        """Convert `seconds` to frame."""
        return int(round(seconds * self._framerate, 0))

    def seconds_to_time(self, seconds):
        """Convert `seconds` to time."""
        sign = ("-" if seconds < 0 else "")
        seconds = abs(round(seconds, 3))
        if seconds > 359999.999:
            return "%s99:59:59.999" % sign
        return "%s%02.0f:%02.0f:%02.0f.%03.0f" % (sign, seconds // 3600,
                                                  (seconds % 3600) // 60,
                                                  int(seconds % 60),
                                                  (seconds % 1) * 1000)

    def time_to_frame(self, time):
        """Convert `time` to frame."""
        seconds = self.time_to_seconds(time)
        return self.seconds_to_frame(seconds)

    def time_to_seconds(self, time):
        """Convert `time` to seconds."""
        coefficient = (-1 if time.startswith("-") else 1)
        time = (time[1:] if time.startswith("-") else time)
        return coefficient * sum((float(time[ :2]) * 3600,
                                  float(time[3:5]) * 60,
                                  float(time[6:8]),
                                  float(time[9: ]) / 1000,))

    def to_frame(self, position):
        """Convert `position` to frame."""
        if isinstance(position, basestring):
            return self.time_to_frame(position)
        if isinstance(position, int):
            return position
        if isinstance(position, float):
            return self.seconds_to_frame(position)
        raise ValueError("Invalid type for position: %s"
                         % repr(type(position)))

    def to_seconds(self, position):
        """Convert `position` to secods."""
        if isinstance(position, basestring):
            return self.time_to_seconds(position)
        if isinstance(position, int):
            return self.frame_to_seconds(position)
        if isinstance(position, float):
            return position
        raise ValueError("Invalid type for position: %s"
                         % repr(type(position)))

    def to_time(self, position):
        """Convert `position` to time."""
        if isinstance(position, basestring):
            return position
        if isinstance(position, int):
            return self.frame_to_time(position)
        if isinstance(position, float):
            return self.seconds_to_time(position)
        raise ValueError("Invalid type for position: %s"
                         % repr(type(position)))



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Reading, writing and manipulating text-based subtitle files.

:mod:`aeidon` is a Python package that provides classes and functions for
dealing with text-based subtitle files of many different formats. Functions
exist for reading and writing subtitle files as well as manipulating subtitle
data, i.e. positions (times or frames) and texts. Three examples of basic use
of the :mod:`aeidon` package follow below.

Converting a file from the SubRip format to the MicroDVD format::

    project = aeidon.Project()
    project.open_main("/path/to/file.srt", "utf_8")
    project.set_framerate(aeidon.framerates.FPS_23_976)
    project.save_main(aeidon.files.new(aeidon.formats.MICRODVD,
                                       "/path/to/file.sub",
                                       "utf_8"))

Making all subtitles in a file appear two seconds earlier::

    project = aeidon.Project()
    project.open_main("/path/to/file.srt", "utf_8")
    project.shift_positions(None, -2.0)
    project.save_main()

Removing all blank subtitles::

    project = aeidon.Project()
    project.open_main("/path/to/file.srt", "utf_8")
    indices = [i for i, x in enumerate(project.subtitles)
               if not x.main_text]

    project.remove_subtitles(indices)
    project.save_main()

:mod:`aeidon` handles positions as either times (as strings of form
``HH:MM:SS.SSS``), frames (as integers) or in some cases seconds (as floats).
All these can be used with functions that edit subtitle data regardless of what
the native position type of the file format used is.

:mod:`aeidon` handles two separate documents that comprise a project -- a main
and a translation document. These correspond to separate files, but the
subtitles are common since positions are shared.

:mod:`aeidon` includes an undo/redo-system. Any subtitle data-editing methods
of :class:`aeidon.Project` (ones marked with the :func:`aeidon.deco.revertable`
decorator) can be undone and redone. If using :mod:`aeidon` in a context where
reverting actions is never needed, greater flexibility can be achieved by
accessing the subtitles directly (via :attr:`aeidon.Project.subtitles`).

:var CONFIG_HOME_DIR: Path to the user's local configuration directory
:var DATA_DIR: Path to the global data directory
:var DATA_HOME_DIR: Path to the user's local data directory
:var LOCALE_DIR: Path to the global locale directory

:var align_methods: Enumerations for subtitle align methods
:var documents: Enumerations for document types
:var formats: Enumerations for subtitle file format types
:var framerates: Enumerations for framerate types
:var modes: Enumerations for position unit types
:var newlines: Enumerations for newline character types
:var players: Enumerations for video player application types
:var registers: Enumerations for action action reversion register types

:var debug: ``True`` to perform additional run-time checks

   `debug` is used in many cases at import time and should thus not be set
   directly, but rather by setting a non-blank value, e.g. "1", to either of
   environment variables ``AEIDON_DEBUG`` or ``GAUPOL_DEBUG``.

:var re_any_tag: Regular expression for markup tags of any format
"""

import os
import re

__version__ = "0.19.2"

debug = (bool(os.environ.get("AEIDON_DEBUG", "")) or
         bool(os.environ.get("GAUPOL_DEBUG", "")))

re_any_tag = re.compile(r"(^[/\\_]+|<.*?>|\{.*?\})")

from aeidon.paths import *
from aeidon import deco
from aeidon import i18n
from aeidon import util
from aeidon import temp
from aeidon.contractual import *
from aeidon.delegate import *
from aeidon.singleton import *
from aeidon.mutables import *
from aeidon.observable import *
from aeidon.errors import *
from aeidon.enum import *
from aeidon.enums.align import *
from aeidon.enums.documents import *
from aeidon.enums.formats import *
from aeidon.enums.framerates import *
from aeidon.enums.modes import *
from aeidon.enums.newlines import *
from aeidon.enums.players import *
from aeidon.enums.registers import *
from aeidon import encodings
from aeidon import languages
from aeidon import countries
from aeidon import locales
from aeidon import scripts
from aeidon.metadata import *
from aeidon.calculator import *
from aeidon.finder import *
from aeidon.parser import *
from aeidon.liner import *
from aeidon import containers
from aeidon.subtitle import *
from aeidon.file import *
from aeidon import files
from aeidon.markup import *
from aeidon import tags
from aeidon.converter import *
from aeidon.pattern import *
from aeidon.patternman import *
from aeidon.clipboard import *
from aeidon.revertable import *
from aeidon import agents
from aeidon.project import *
from aeidon.unittest import *



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for selecting a subtitle file to save."""

import aeidon
import gaupol
import gtk
import os

__all__ = ("SaveDialog",)


class SaveDialog(gaupol.FileDialog):

    """Dialog for selecting a subtitle file to save."""

    _widgets = ("encoding_combo", "format_combo", "newline_combo")

    def __init__(self, parent, title):
        """Initialize a :class:`SaveDialog` object."""
        gaupol.FileDialog.__init__(self, "save-dialog.ui")
        self._init_filters()
        self._init_format_combo()
        self._init_encoding_combo()
        self._init_newline_combo()
        self._init_values()
        self.set_title(title)
        self.set_transient_for(parent)

    def _init_format_combo(self):
        """Initialize the format combo box."""
        store = gtk.ListStore(str)
        self._format_combo.set_model(store)
        for name in (x.label for x in aeidon.formats):
            store.append((name,))
        view = self._format_combo.get_child()
        view.set_displayed_row(0)
        renderer = gtk.CellRendererText()
        self._format_combo.pack_start(renderer, True)
        self._format_combo.add_attribute(renderer, "text", 0)

    def _init_newline_combo(self):
        """Initialize the newline combo box."""
        store = gtk.ListStore(str)
        self._newline_combo.set_model(store)
        for name in (x.label for x in aeidon.newlines):
            store.append((name,))
        view = self._newline_combo.get_child()
        view.set_displayed_row(0)
        renderer = gtk.CellRendererText()
        self._newline_combo.pack_start(renderer, True)
        self._newline_combo.add_attribute(renderer, "text", 0)

    def _init_values(self):
        """Initialize default values for widgets."""
        if os.path.isdir(gaupol.conf.file.directory):
            self.set_current_folder(gaupol.conf.file.directory)
        self.set_encoding(gaupol.conf.file.encoding)
        self.set_format(gaupol.conf.file.format)
        self.set_newline(gaupol.conf.file.newline)

    def _on_format_combo_changed(self, combo_box):
        """Change the extension of the current filename."""
        path = self.get_filename()
        if path is None: return
        self.unselect_filename(path)
        dirname = os.path.dirname(path)
        basename = os.path.basename(path)
        format = self.get_format()
        basename = aeidon.util.replace_extension(basename, format)
        path = os.path.join(dirname, basename)
        self.set_current_name(basename)
        self.set_filename(path)

    def _on_response(self, dialog, response):
        """Save default values for widgets."""
        gaupol.conf.file.encoding = self.get_encoding()
        gaupol.conf.file.format = self.get_format()
        gaupol.conf.file.newline = self.get_newline()
        directory = self.get_current_folder()
        if directory is not None:
            gaupol.conf.file.directory = directory

    def _on_save_button_event(self, button, event):
        """Ensure that the filename contains an extension."""
        # Catch all events on save button to ensure that all mouse and keyboard
        # events are handled and that a possibly lacking extension is added to
        # the filename so that overwrite confirmation check is done correctly.
        self._format_combo.emit("changed")

    def get_format(self):
        """Return the selected format."""
        index = self._format_combo.get_active()
        return aeidon.formats[index]

    def get_newline(self):
        """Return the selected newline."""
        index = self._newline_combo.get_active()
        return aeidon.newlines[index]

    def set_format(self, format):
        """Set the selected format."""
        if format is None: return
        self._format_combo.set_active(format)

    def set_name(self, path):
        """Set the selected filename."""
        if os.path.isfile(path):
            return self.set_filename(path)
        return self.set_current_name(path)

    def set_newline(self, newline):
        """Set the selected newline."""
        if newline is None: return
        self._newline_combo.set_active(newline)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestModule(aeidon.TestCase):

    def test_code_to_name(self):
        name = aeidon.i18n.dgettext("iso_3166", "Ireland")
        assert aeidon.countries.code_to_name("IE") == name
        name = aeidon.i18n.dgettext("iso_3166", "Yemen")
        assert aeidon.countries.code_to_name("YE") == name

    def test_code_to_name__key_error(self):
        self.raises(KeyError, aeidon.countries.code_to_name, "XX")

    def test_is_valid(self):
        assert aeidon.countries.is_valid("RU")
        assert not aeidon.countries.is_valid("XX")



# Copyright (C) 2007 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Base class for single-instance classes."""

__all__ = ("Singleton",)


class Singleton(object):

    """Base class for single-instance classes."""

    def __new__(cls, *args, **kwargs):
        """Return possibly existing instance."""
        if "_instance" not in cls.__dict__:
            cls._instance = object.__new__(cls, *args, **kwargs)
        return cls._instance



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Base class for unit test cases."""

import aeidon
import os

__all__ = ("TestCase",)


class TestCase(object):

    """Base class for unit test cases.

    Unit tests are designed to be run with ``py.test``, ``nose`` or something
    compatible. Tests should use plain ``assert`` statements to allow multiple
    different tools to be used to run the tests.
    """

    __metaclass__ = aeidon.Contractual

    def get_sample_text_ensure(self, value, format, name=None):
        assert value.count("a") > 10

    @aeidon.deco.memoize
    def get_sample_text(self, format, name=None):
        """Return sample text for subtitle file `format`."""
        name = name or format.name.lower()
        basename = "".join((name, format.extension))
        path = os.path.join(aeidon.DATA_DIR, "samples", basename)
        with open(path, "r") as fobj:
            return fobj.read().strip()

    def new_microdvd_file_ensure(self, value):
        assert os.path.isfile(value)

    def new_microdvd_file(self):
        """Return path to a new temporary ``MicroDVD`` file."""
        return self.new_temp_file(aeidon.formats.MICRODVD)

    def new_project_ensure(self, value):
        assert value.subtitles

    def new_project(self):
        """Return a new project with both main and translation files."""
        project = aeidon.Project()
        project.open_main(self.new_subrip_file(), "ascii")
        project.open_translation(self.new_microdvd_file(), "ascii")
        return project

    def new_subrip_file_ensure(self, value):
        assert os.path.isfile(value)

    def new_subrip_file(self):
        """Return path to a new temporary ``SubRip`` file."""
        return self.new_temp_file(aeidon.formats.SUBRIP)

    def new_temp_file_ensure(self, value, format, name=None):
        assert os.path.isfile(value)

    def new_temp_file(self, format, name=None):
        """Return path to a new temporary subtitle file."""
        text = self.get_sample_text(format, name)
        path = aeidon.temp.create(format.extension)
        aeidon.temp.close(path)
        with open(path, "w") as fobj:
            fobj.write(text)
        return path

    def raises(self, exception, function, *args, **kwargs):
        """Assert that calling `function` raises `exception`."""
        try:
            function(*args, **kwargs)
        except exception:
            return
        raise AssertionError("Function '%s' failed to raise exception '%s'"
                             % (repr(function), repr(exception)))

    def setUp(self):
        """Compatibility alias for :meth:`setup_method`."""
        self.setup_method(None)

    def setup_method(self, method):
        """Set state for executing tests in `method`."""
        pass

    def tearDown(self):
        """Compatibility alias for :meth:`teardown_method`."""
        self.teardown_method(None)

    def teardown_method(self, method):
        """Remove state set for executing tests in `method`."""
        pass

    def test___init__(self):
        """Make sure that :meth:`setup_method` is always run."""
        pass



# Copyright (C) 2009 Osmo Salomaa
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import re

def on_autodoc_process_docstring(app, what, name, obj, options, lines):
    """Reformat inline variable markup (':[ci]?var:')."""
    re_private = re.compile(r"^:[ci]?var _(\w+):")
    lines[:] = [x for x in lines if not re_private.search(x)]
    parent = (name.split(".")[-1] if what == "class" else name)
    pattern = r"^:[ci]?var +(\w+): +(.*?)$"
    replacement = r".. attribute:: %s.\1\n\n   \2\n" % parent
    re_var = re.compile(pattern, re.MULTILINE)
    text = re_var.sub(replacement, "\n".join(lines))
    lines[:] = text.split("\n")

def setup(app):
    app.connect("autodoc-process-docstring",
                on_autodoc_process_docstring)

    # Don't add the directive header to classes.
    from sphinx.ext.autodoc import ClassDocumenter
    ClassDocumenter.add_directive_header = lambda *args: None



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for configuring spell-check."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("LanguageDialog",)


class LanguageDialog(gaupol.BuilderDialog):

    """Dialog for configuring spell-check."""

    __metaclass__ = aeidon.Contractual

    _widgets = ("all_radio",
                "current_radio",
                "language_alignment",
                "language_title_label",
                "main_radio",
                "target_vbox",
                "tran_radio",
                "tree_view",)

    def __init___require(self, parent, show_target=True):
        assert aeidon.util.enchant_available()

    def __init__(self, parent, show_target=True):
        """Initialize a :class:`LanguageDialog` object."""
        gaupol.BuilderDialog.__init__(self, "language-dialog.ui")
        self._init_visibilities(show_target)
        self._init_tree_view()
        self._init_values()
        gaupol.util.scale_to_content(self._tree_view,
                                     min_nchar=10,
                                     min_nlines=5,
                                     max_nchar=80,
                                     max_nlines=15)

        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_CLOSE)

    def _init_tree_view(self):
        """Initialize the tree view."""
        selection = self._tree_view.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        selection.connect("changed", self._on_tree_view_selection_changed)
        store = gtk.ListStore(str, str)
        self._populate_store(store)
        store.set_sort_column_id(1, gtk.SORT_ASCENDING)
        self._tree_view.set_model(store)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("", renderer, text=1)
        column.set_sort_column_id(1)
        self._tree_view.append_column(column)

    def _init_values(self):
        """Initialize default values for widgets."""
        store = self._tree_view.get_model()
        selection = self._tree_view.get_selection()
        for i in range(len(store)):
            if store[i][0] == gaupol.conf.spell_check.language:
                selection.select_path(i)
        field = gaupol.conf.spell_check.field
        target = gaupol.conf.spell_check.target
        self._main_radio.set_active(field == gaupol.fields.MAIN_TEXT)
        self._tran_radio.set_active(field == gaupol.fields.TRAN_TEXT)
        self._all_radio.set_active(target == gaupol.targets.ALL)
        self._current_radio.set_active(target == gaupol.targets.CURRENT)

    def _init_visibilities(self, show_target):
        """Initialize visibilities of target widgets."""
        if not show_target:
            self._language_title_label.hide()
            self._target_vbox.hide()
            self._language_alignment.set_padding(0, 0, 0, 0)
            self._dialog.set_title(_("Set Language"))

    def _on_all_radio_toggled(self, radio_button):
        """Save the selected target."""
        target = (gaupol.targets.ALL
                  if radio_button.get_active() else
                  gaupol.targets.CURRENT)

        gaupol.conf.spell_check.target = target

    def _on_current_radio_toggled(self, radio_button):
        """Save the selected target."""
        target = (gaupol.targets.CURRENT
                  if radio_button.get_active() else
                  gaupol.targets.ALL)

        gaupol.conf.spell_check.target = target

    def _on_main_radio_toggled(self, radio_button):
        """Save the selected field."""
        field = (gaupol.fields.MAIN_TEXT
                 if radio_button.get_active() else
                 gaupol.fields.TRAN_TEXT)

        gaupol.conf.spell_check.field = field

    def _on_tran_radio_toggled(self, radio_button):
        """Save the selected field."""
        field = (gaupol.fields.TRAN_TEXT
                 if radio_button.get_active() else
                 gaupol.fields.MAIN_TEXT)

        gaupol.conf.spell_check.field = field

    def _on_tree_view_selection_changed(self, selection):
        """Save the selected language."""
        store, itr = selection.get_selected()
        if itr is None: return
        value = store.get_value(itr, 0)
        gaupol.conf.spell_check.language = value

    def _populate_store(self, store):
        """Add all available languages to `store`."""
        import enchant
        try: locales = set(enchant.list_languages())
        except enchant.Error: return
        for locale in locales:
            try: enchant.Dict(locale).check("1")
            except enchant.Error: continue
            try: name = aeidon.locales.code_to_name(locale)
            except LookupError: name = locale
            store.append((locale, name))



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import re


class TestParser(aeidon.TestCase):

    def setup_method(self, method):
        clean_func = lambda x: x.strip()
        self.parser = aeidon.Parser(re.compile(r"<.+?>"), clean_func)

    def test_get_text__margins(self):
        text = ("<i>You will find allies there,</i>\n"
                "<i>and they will pay more.</i>")

        self.parser.set_text(text)
        assert self.parser.get_text() == text

    def test_get_text__tags(self):
        text = ("<i>You will find allies there,</i>\n"
                "<i>and</i> they <i>will pay more.</i>")

        self.parser.set_text(text)
        assert self.parser.get_text() == text

    def test_replace_all__regex(self):
        text = ("<i>One only risks it, <b>because</b>\n"
                "one's survival depends on it.</i>")

        cases = ((r"^", r"--",
                  ("<i>--One only risks it, <b>because</b>\n"
                   "--one's survival depends on it.</i>")),
                 (r"\A", r"--",
                  ("<i>--One only risks it, <b>because</b>\n"
                   "one's survival depends on it.</i>")),
                 (r"$", r"--",
                  ("<i>One only risks it, <b>because--</b>\n"
                   "one's survival depends on it.--</i>")),
                 (r"\Z", r"--",
                  ("<i>One only risks it, <b>because</b>\n"
                   "one's survival depends on it.--</i>")),
                 (r"\s", r"",
                  ("<i>Oneonlyrisksit,<b>because</b>"
                   "one'ssurvivaldependsonit.</i>")),
                 (r"\b", r"-",
                  ("<i>-One- -only- -risks- -it-, <b>-because-</b>\n"
                   "-one-'-s- -survival- -depends- -on- -it-.</i>")),
                 (r"\b$", r"-",
                  ("<i>One only risks it, <b>because-</b>\n"
                   "one's survival depends on it.</i>")),
                 (r"(\w)\w{3}", r"\1\1\1\1",
                  ("<i>One oooo rrrrs it, <b>bbbbuse</b>\n"
                   "one's ssssiiii ddddnds on it.</i>")),
                 (r"it, be", r"'",
                  ("<i>One only risks <b>'cause</b>\n"
                   "one's survival depends on it.</i>")),
                 (r".", r"",
                  ""),)

        for pattern, replacement, new_text in cases:
            self.parser = aeidon.Parser(re.compile(r"<.+?>"))
            self.parser.set_text(text)
            self.parser.set_regex(pattern)
            self.parser.replacement = replacement
            self.parser.replace_all()
            assert self.parser.get_text() == new_text

    def test_replace_all__string(self):
        text = ("<i>One only risks it, <b>because</b>\n"
                "one's survival depends on it.</i>")

        cases = (("i", "-",
                  ("<i>One only r-sks -t, <b>because</b>\n"
                   "one's surv-val depends on -t.</i>")),
                 ("O", "--",
                  ("<i>--ne only risks it, <b>because</b>\n"
                   "one's survival depends on it.</i>")),
                 ("O", "",
                  ("<i>ne only risks it, <b>because</b>\n"
                   "one's survival depends on it.</i>")),
                 ("b", "--",
                  ("<i>One only risks it, <b>--ecause</b>\n"
                   "one's survival depends on it.</i>")),
                 ("b", "",
                  ("<i>One only risks it, <b>ecause</b>\n"
                   "one's survival depends on it.</i>")),
                 ("e", "--",
                  ("<i>On-- only risks it, <b>b--caus--</b>\n"
                   "on--'s survival d--p--nds on it.</i>")),
                 ("e", "",
                  ("<i>On only risks it, <b>bcaus</b>\n"
                   "on's survival dpnds on it.</i>")),
                 ("o", "--",
                  ("<i>One --nly risks it, <b>because</b>\n"
                   "--ne's survival depends --n it.</i>")),
                 ("o", "",
                  ("<i>One nly risks it, <b>because</b>\n"
                   "ne's survival depends n it.</i>")),
                 (".", "--",
                  ("<i>One only risks it, <b>because</b>\n"
                   "one's survival depends on it--</i>")),
                 (".", "",
                  ("<i>One only risks it, <b>because</b>\n"
                   "one's survival depends on it</i>")),)

        for pattern, replacement, new_text in cases:
            self.parser = aeidon.Parser(re.compile(r"<.+?>"))
            self.parser.set_text(text)
            self.parser.pattern = pattern
            self.parser.replacement = replacement
            self.parser.replace_all()
            assert self.parser.get_text() == new_text

    def test_set_text__margins(self):
        text = ("<i>One only risks it, because</i>\n"
                "<i>one's survival depends on it.</i>")

        self.parser.set_text(text)
        assert self.parser.text == (
            "One only risks it, because\n"
            "one's survival depends on it.")

    def test_set_text__tags(self):
        text = ("<i>One only risks it, <b>because</b>\n"
                "one's survival depends on it.</i>")

        self.parser.set_text(text)
        assert self.parser.text == (
            "One only risks it, because\n"
            "one's survival depends on it.")



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Base class for subtitle files."""

import aeidon
import codecs
import contextlib
import os
import re

__all__ = ("SubtitleFile",)


class SubtitleFile(object):

    """Base class for subtitle files.

    :cvar format: :attr:`aeidon.formats` item corresponding to file format
    :cvar mode: :attr:`aeidon.modes` item corresponding to the native positions
    :ivar encoding: Character encoding used to read and write file
    :ivar has_utf_16_bom: True if BOM found for UTF-16-BE or UTF-16-LE
    :ivar header: String of generic information at the top of the file
    :ivar newline: :attr:`aeidon.newlines` item, detected upon read
    :ivar path: Full, absolute path to the file on disk

    If the file format contains a header, it will default to a fairly blank
    template header read upon instantiation of the class, from either
    ``aeidon.DATA_DIR/headers`` or ``aeidon.DATA_HOME_DIR/headers``. If the
    read file contains a header, it will replace the template.
    """

    __metaclass__ = aeidon.Contractual
    format = aeidon.formats.NONE
    mode = aeidon.modes.NONE

    def __init__(self, path, encoding, newline=None):
        """Initialize a :class:`SubtitleFile` object."""
        self.encoding = encoding
        self.has_utf_16_bom = False
        self.header = (aeidon.util.get_template_header(self.format)
                       if self.format.has_header else "")

        self.newline = newline or aeidon.util.get_default_newline()
        self.path = os.path.abspath(path)

    def _get_subtitle(self):
        """Return a new subtitle instance with proper properties."""
        return aeidon.Subtitle(self.mode)

    def _invariant(self):
        assert aeidon.encodings.is_valid_code(self.encoding)

    def _read_lines(self):
        """Read file to a unicoded list of lines.

        All newlines are stripped.
        All blank lines from beginning and end are removed.
        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        Return a list of lines read.
        """
        re_newline_char = re.compile(r"\r?\n?$")
        args = (self.path, "r", self.encoding)
        with contextlib.closing(codecs.open(*args)) as fobj:
            lines = fobj.readlines()
            lines = [re_newline_char.sub("", x) for x in lines]
        for index in (0, -1):
            while lines and (not lines[index].strip()):
                lines.pop(index)
        newline = aeidon.util.detect_newlines(self.path)
        if newline is not None:
            self.newline = newline
        if self.encoding == "utf_8":
            bom = unicode(codecs.BOM_UTF8, "utf_8")
            if lines and lines[0].startswith(bom):
                # If a UTF-8 BOM (a.k.a. signature) is found, reread file with
                # UTF-8-SIG encoding, which automatically strips the BOM when
                # reading and adds it when writing.
                self.encoding = "utf_8_sig"
                return SubtitleFile._read_lines(self)
        if self.encoding.startswith("utf_16"):
            # Python automatically strips the UTF-16 BOM when reading, but only
            # when using UTF-16. If using UTF-16-BE or UTF-16-LE, the BOM is
            # kept at the beginning of the first line. It is read correctly, so
            # it should FE FF for both BE and LE.
            bom = unicode(codecs.BOM_UTF16_BE, "utf_16_be")
            if lines and lines[0].startswith(bom):
                self.has_utf_16_bom = True
                lines[0] = lines[0].replace(bom, "")
            # Handle erroneous (?) UTF-16 encoded subtitles that use
            # NULL-character filled linebreaks '\x00\r\x00\n', which
            # readlines interprets as two separate linebreaks.
            if not any(lines[i] for i in range(1, len(lines), 2)):
                lines = [lines[i] for i in range(0, len(lines), 2)]
        return lines

    def copy_from_require(self, other):
        assert isinstance(other, SubtitleFile)

    def copy_from(self, other):
        """Copy generic properties from `other`."""
        if self.format == other.format:
            self.header = other.header
        self.has_utf_16_bom = other.has_utf_16_bom

    def read(self):
        """Read file and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        """
        raise NotImplementedError

    def write_require(self, subtitles, doc):
        assert self.newline is not None

    def write(self, subtitles, doc):
        """Write `subtitles` from `doc` to file.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        args = (self.path, "w", self.encoding)
        with contextlib.closing(codecs.open(*args)) as fobj:
            # UTF-8-SIG automatically adds the UTF-8 signature BOM. Likewise,
            # UTF-16 automatically adds the system default BOM, but
            # UTF-16-BE and UTF-16-LE don't. For the latter two, add the BOM,
            # if it was originally read in the file.
            if self.has_utf_16_bom and (self.encoding == "utf_16_be"):
                fobj.write(unicode(codecs.BOM_UTF16_BE, "utf_16_be"))
            if self.has_utf_16_bom and (self.encoding == "utf_16_le"):
                fobj.write(unicode(codecs.BOM_UTF16_LE, "utf_16_le"))
            self.write_to_file(subtitles, doc, fobj)

    def write_to_file_require(self, subtitles, doc, fobj):
        assert hasattr(fobj, "write")
        assert self.newline is not None

    def write_to_file(self, subtitles, doc, fobj):
        """Write `subtitles` from `doc` to `fobj`.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        raise NotImplementedError



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Names and ISO 639 codes for languages and conversions between them."""

import aeidon
import os
import re

_languages = {}


def _init_languages():
    """Initialize the dictionary mapping codes to names."""
    import xml.etree.ElementTree as ET
    path = "/usr/share/xml/iso-codes/iso_639.xml"
    if not os.path.isfile(path):
        # Use local, possibly outdated copy, only as a fallback.
        path = os.path.join(aeidon.DATA_DIR, "iso-codes", "iso_639.xml")
    for element in ET.parse(path).findall("iso_639_entry"):
        code = element.get("iso_639_1_code")
        name = element.get("name")
        if code and name:
            _languages[code] = name

def code_to_name_require(code):
    assert re.match(r"^[a-z][a-z]$", code)

@aeidon.deco.contractual
def code_to_name(code):
    """Convert ISO 639 `code` to localized language name.

    Raise :exc:`KeyError` if `code` not found.
    """
    return aeidon.i18n.dgettext("iso_639", _languages[code])

def is_valid_require(code):
    assert re.match(r"^[a-z][a-z]$", code)

@aeidon.deco.contractual
def is_valid(code):
    """Return ``True`` if `code` is a valid ISO 639 language code."""
    return (code in _languages)


_init_languages()



# Copyright (C) 2005-2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Editing subtitle data."""

import aeidon
import gaupol
_ = aeidon.i18n._


class EditAgent(aeidon.Delegate):

    """Editing subtitle data."""

    __metaclass__ = aeidon.Contractual

    def __init__(self, master):
        """Initialize an :class:`EditAgent` object."""
        aeidon.Delegate.__init__(self, master)
        self._pref_dialog = None

    def _on_pref_dialog_response(self, *args):
        """Destroy the preferences dialog."""
        self._pref_dialog.destroy()
        self._pref_dialog = None
        gaupol.conf.write_to_file()

    def _set_unsafe_sensitivities(self, sensitive):
        """Set sensitivities of unsafe UI manager actions."""
        action_group = self.get_action_group("main-unsafe")
        action_group.set_sensitive(sensitive)

    def _sync_clipboards(self, page):
        """Synchronize all clipboards to match that of `page`."""
        texts = page.project.clipboard.get_texts()
        self.clipboard.set_texts(texts)
        for item in self.pages:
            item.project.clipboard.set_texts(texts)
        text = page.project.clipboard.get_string()
        self.x_clipboard.set_text(text)
        self.update_gui()

    @aeidon.deco.export
    def _on_clear_texts_activate(self, *args):
        """Clear the selected texts."""
        page = self.get_current_page()
        rows = page.view.get_selected_rows()
        col = page.view.get_focus()[1]
        doc = page.text_column_to_document(col)
        page.project.clear_texts(rows, doc)

    @aeidon.deco.export
    def _on_copy_texts_activate(self, *args):
        """Copy the selected texts to the clipboard."""
        page = self.get_current_page()
        rows = page.view.get_selected_rows()
        col = page.view.get_focus()[1]
        doc = page.text_column_to_document(col)
        page.project.copy_texts(rows, doc)
        self._sync_clipboards(page)

    @aeidon.deco.export
    def _on_cut_texts_activate(self, *args):
        """Cut the selected texts to the clipboard."""
        page = self.get_current_page()
        rows = page.view.get_selected_rows()
        col = page.view.get_focus()[1]
        doc = page.text_column_to_document(col)
        page.project.cut_texts(rows, doc)
        self._sync_clipboards(page)

    @aeidon.deco.export
    def _on_edit_headers_activate(self, *args):
        """Edit file headers."""
        dialog = gaupol.HeaderDialog(self.window, self)
        gaupol.util.flash_dialog(dialog)

    @aeidon.deco.export
    def _on_edit_next_value_activate(self, *args):
        """Edit the focused column of the next subtitle."""
        page = self.get_current_page()
        path, column = page.view.get_cursor()
        page.view.set_cursor((path[0] + 1,), column, True)

    @aeidon.deco.export
    def _on_edit_preferences_activate(self, *args):
        """Configure Gaupol."""
        if self._pref_dialog is not None:
            return self._pref_dialog.present()
        self._pref_dialog = gaupol.PreferencesDialog(self.window, self)
        aeidon.util.connect(self, "_pref_dialog", "response")
        self._pref_dialog.show()

    @aeidon.deco.export
    def _on_edit_value_activate(self, *args):
        """Edit the focused cell."""
        page = self.get_current_page()
        row, column = page.view.get_cursor()
        page.view.set_cursor(row, column, True)

    @aeidon.deco.export
    def _on_extend_selection_to_beginning_activate(self, *args):
        """Extend the selection up to the first subtitle."""
        page = self.get_current_page()
        row = page.view.get_selected_rows()[-1]
        rows = range(0, row + 1)
        page.view.select_rows(rows)

    @aeidon.deco.export
    def _on_extend_selection_to_end_activate(self, *args):
        """Extend the selection up to the last subtitle."""
        page = self.get_current_page()
        row = page.view.get_selected_rows()[0]
        rows = range(row, len(page.project.subtitles))
        page.view.select_rows(rows)

    @aeidon.deco.export
    def _on_insert_subtitles_activate(self, *args):
        """Insert subtitles."""
        dialog = gaupol.InsertDialog(self.window, self)
        gaupol.util.flash_dialog(dialog)

    @aeidon.deco.export
    def _on_invert_selection_activate(self, *args):
        """Invert the current selection."""
        page = self.get_current_page()
        rows = set(range(0, len(page.project.subtitles)))
        rows -= set(page.view.get_selected_rows())
        page.view.select_rows(rows)

    @aeidon.deco.export
    def _on_merge_subtitles_activate(self, *args):
        """Merge the selected subtitles."""
        page = self.get_current_page()
        rows = page.view.get_selected_rows()
        page.project.merge_subtitles(rows)

    @aeidon.deco.export
    def _on_paste_texts_activate(self, *args):
        """Paste texts from the clipboard."""
        page = self.get_current_page()
        rows = page.view.get_selected_rows()
        row, col = page.view.get_focus()
        doc = page.text_column_to_document(col)
        length = len(page.project.subtitles)
        # Ensure that even if new subtitles need to be inserted,
        # focus and scroll position are not moved to the end.
        rect = page.view.get_visible_rect()
        window = page.view.get_bin_window()
        window.freeze_updates()
        rows = page.project.paste_texts(rows[0], doc)
        rows = page.view.get_selected_rows()
        page.view.set_focus(row, col)
        page.view.select_rows(rows)
        page.view.scroll_to_point(rect.x, rect.y)
        window.thaw_updates()
        count = len(page.project.subtitles) - length
        if count <= 0: return
        self.flash_message(aeidon.i18n.ngettext(
                "Inserted %d subtitle to fit clipboard contents",
                "Inserted %d subtitles to fit clipboard contents",
                count) % count)

    @aeidon.deco.export
    def _on_project_action_done(self, *args):
        """Update user interface and send a signal."""
        page = self.get_current_page()
        self.update_gui()
        self.emit("page-changed", page)

    @aeidon.deco.export
    def _on_project_action_redone(self, *args):
        """Update user interface and send a signal."""
        page = self.get_current_page()
        row = page.view.get_focus()[0]
        if row is not None:
            page.view.scroll_to_row(row)
        self.update_gui()
        self.emit("page-changed", page)

    @aeidon.deco.export
    def _on_project_action_undone(self, *args):
        """Update user interface and send a signal."""
        page = self.get_current_page()
        row = page.view.get_focus()[0]
        if row is not None:
            page.view.scroll_to_row(row)
        self.update_gui()
        self.emit("page-changed", page)

    @aeidon.deco.export
    def _on_redo_action_activate(self, *args):
        """Redo the last undone action."""
        self.redo()

    @aeidon.deco.export
    def _on_remove_subtitles_activate(self, *args):
        """Remove the selected subtitles."""
        page = self.get_current_page()
        rows = page.view.get_selected_rows()
        page.project.remove_subtitles(rows)

    @aeidon.deco.export
    def _on_select_all_activate(self, *args):
        """Select all subtitles."""
        page = self.get_current_page()
        selection = page.view.get_selection()
        selection.select_all()

    @aeidon.deco.export
    def _on_split_subtitle_activate(self, *args):
        """Split the selected subtitle."""
        page = self.get_current_page()
        row = page.view.get_selected_rows()[0]
        page.project.split_subtitle(row)

    @aeidon.deco.export
    def _on_undo_action_activate(self, *args):
        """Undo the last action."""
        self.undo()

    @aeidon.deco.export
    def _on_view_renderer_edited(self, renderer, path, value, column):
        """Save changes made while editing cell."""
        self._set_unsafe_sensitivities(True)
        self.push_message(None)
        page = self.get_current_page()
        row = int(path)
        col = page.view.get_columns().index(column)
        if page.view.is_position_column(col):
            if not value: return
            if page.edit_mode == aeidon.modes.FRAME:
                try: value = int(value)
                except ValueError: return
        if col == page.view.columns.START:
            return page.project.set_start(row, value)
        if col == page.view.columns.END:
            return page.project.set_end(row, value)
        if col ==  page.view.columns.DURATION:
            if page.edit_mode == aeidon.modes.TIME:
                # pylint: disable=E1103
                value = value.replace(",", ".")
                try: value = float(value)
                except ValueError: return
            return page.project.set_duration(row, value)
        doc = page.text_column_to_document(col)
        page.project.set_text(row, doc, value)

    @aeidon.deco.export
    def _on_view_renderer_editing_canceled(self, *args):
        """Unset state set for editing cell."""
        self._set_unsafe_sensitivities(True)
        self.push_message(None)

    @aeidon.deco.export
    def _on_view_renderer_editing_started(self,
                                          renderer,
                                          editor,
                                          path,
                                          column):

        """Set proper state for editing cell."""
        self._set_unsafe_sensitivities(False)
        page = self.get_current_page()
        col = page.view.get_columns().index(column)
        if not page.view.is_text_column(col): return
        self.push_message(_("Use Shift+Return for line-break"))

    def redo_require(self, count=1):
        page = self.get_current_page()
        assert page.project.can_redo()

    @aeidon.deco.export
    def redo(self, count=1):
        """Redo `count` amount of actions."""
        gaupol.util.set_cursor_busy(self.window)
        page = self.get_current_page()
        page.project.redo(count)
        gaupol.util.set_cursor_normal(self.window)

    def undo_require(self, count=1):
        page = self.get_current_page()
        assert page.project.can_undo()

    @aeidon.deco.export
    def undo(self, count=1):
        """Undo `count` amount of actions."""
        gaupol.util.set_cursor_busy(self.window)
        page = self.get_current_page()
        page.project.undo(count)
        gaupol.util.set_cursor_normal(self.window)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Editing times and frames."""

import aeidon
import gaupol


class PositionAgent(aeidon.Delegate):

    """Editing times and frames."""

    @aeidon.deco.export
    def _on_adjust_durations_activate(self, *args):
        """Lengthen or shorten durations."""
        dialog = gaupol.DurationAdjustDialog(self.window, self)
        gaupol.util.flash_dialog(dialog)

    @aeidon.deco.export
    def _on_convert_framerate_activate(self, *args):
        """Convert framerate."""
        dialog = gaupol.FramerateConvertDialog(self.window, self)
        gaupol.util.flash_dialog(dialog)

    @aeidon.deco.export
    def _on_shift_positions_activate(self, *args):
        """Make subtitles appear earlier or later."""
        page = self.get_current_page()
        if page.edit_mode == aeidon.modes.TIME:
            dialog = gaupol.TimeShiftDialog(self.window, self)
        if page.edit_mode == aeidon.modes.FRAME:
            dialog = gaupol.FrameShiftDialog(self.window, self)
        gaupol.util.flash_dialog(dialog)

    @aeidon.deco.export
    def _on_speech_recognition_activate(self, *args):
        """Generate subtitles by voice and speech recognition."""
        dialog = gaupol.SpeechRecognitionDialog(self.window, self)
        gaupol.util.flash_dialog(dialog)

    @aeidon.deco.export
    def _on_transform_positions_activate(self, *args):
        """Change positions by linear two-point correction."""
        page = self.get_current_page()
        if page.edit_mode == aeidon.modes.TIME:
            dialog = gaupol.TimeTransformDialog(self.window, self)
        if page.edit_mode == aeidon.modes.FRAME:
            dialog = gaupol.FrameTransformDialog(self.window, self)
        gaupol.util.flash_dialog(dialog)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Advanced Sub Station Alpha file."""

import aeidon

__all__ = ("AdvSubStationAlpha",)


class AdvSubStationAlpha(aeidon.files.SubStationAlpha):

    """Advanced Sub Station Alpha file.

    :ivar event_fields: Tuple of field names for the ``[Events]`` section
    """

    format = aeidon.formats.ASS

    def __init__(self, path, encoding, newline=None):
        """Initialize an :class:`AdvSubStationAlpha` object."""
        aeidon.files.SubStationAlpha.__init__(self, path, encoding, newline)
        self.event_fields = ("Layer",
                             "Start",
                             "End",
                             "Style",
                             "Name",
                             "MarginL",
                             "MarginR",
                             "MarginV",
                             "Effect",
                             "Text",)

    def _decode_field(self, field_name, value, subtitle):
        """Save `value` of field as a subtitle attribute."""
        if field_name == "Layer":
            return setattr(subtitle.ssa, "layer", int(value))
        decode = aeidon.files.SubStationAlpha._decode_field
        return decode(self, field_name, value, subtitle)

    def _encode_field(self, field_name, subtitle, doc):
        """Return value of field as string to be written to file."""
        if field_name == "Layer":
            return str(subtitle.ssa.layer)
        encode = aeidon.files.SubStationAlpha._encode_field
        return encode(self, field_name, subtitle, doc)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import codecs
import sys


# pylint: disable=W0223
class PuppetSubtitleFile(aeidon.SubtitleFile):

    format = aeidon.formats.SUBVIEWER2
    mode = aeidon.modes.TIME


class TestSubtitleFile(aeidon.TestCase):

    def setup_method(self, method):
        path = self.new_temp_file(PuppetSubtitleFile.format)
        newline = aeidon.newlines.UNIX
        self.file = PuppetSubtitleFile(path, "ascii", newline)

    def test__read_lines(self):
        with open(self.file.path, "a") as fobj:
            fobj.write("\n\r\n\n")
        lines = self.file._read_lines()
        assert lines
        for line in lines:
            assert not line.endswith("\n")
        assert lines[-1]

    def test_copy_from(self):
        self.file.has_utf_16_bom = False
        self.file.header = "test"
        format = aeidon.formats.SUBVIEWER2
        path = self.new_temp_file(PuppetSubtitleFile.format)
        new_file = PuppetSubtitleFile(path, "ascii")
        new_file.copy_from(self.file)
        assert new_file.has_utf_16_bom == False
        assert new_file.header == "test"

    def test_read(self):
        self.raises(NotImplementedError,
                    self.file.read)

    def test_read__utf_16(self):
        path = self.new_subrip_file()
        with open(path, "r") as fobj:
            text = fobj.read()
        with codecs.open(path, "w", "utf_16") as fobj:
            fobj.write(text)
        sfile = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_16")
        sfile.read()

    def test_read__utf_16_be(self):
        path = self.new_subrip_file()
        with open(path, "r") as fobj:
            text = fobj.read()
        with codecs.open(path, "w", "utf_16_be") as fobj:
            fobj.write(unicode(codecs.BOM_UTF16_BE, "utf_16_be"))
            fobj.write(text)
        sfile = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_16_be")
        sfile.read()

    def test_read__utf_16_le(self):
        path = self.new_subrip_file()
        with open(path, "r") as fobj:
            text = fobj.read()
        with codecs.open(path, "w", "utf_16_le") as fobj:
            fobj.write(unicode(codecs.BOM_UTF16_LE, "utf_16_le"))
            fobj.write(text)
        sfile = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_16_le")
        sfile.read()

    def test_read__utf_8_sig(self):
        path = self.new_subrip_file()
        with open(path, "r") as fobj:
            text = fobj.read()
        with codecs.open(path, "w", "utf_8_sig") as fobj:
            fobj.write(text)
        sfile = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_8_sig")
        sfile.read()

    def test_write(self):
        self.raises(NotImplementedError,
                    self.file.write,
                    (), aeidon.documents.MAIN)

    def test_write_to_file(self):
        self.raises(NotImplementedError,
                    self.file.write_to_file,
                    (), aeidon.documents.MAIN, sys.stdout)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestSaveAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()
        self.delegate = self.project.save_main.im_self

    def test__copy_file(self):
        copy_file = self.delegate._copy_file
        source = self.new_subrip_file()
        destination = self.new_subrip_file()
        assert copy_file(source, destination)
        assert not copy_file(source, "/////")

    def test__move_file(self):
        move_file = self.delegate._move_file
        source = self.new_subrip_file()
        destination = self.new_subrip_file()
        assert move_file(source, destination)
        assert not move_file(source, "/////")

    def test__remove_file(self):
        remove_file = self.delegate._remove_file
        path = self.new_subrip_file()
        assert remove_file(path)
        assert not remove_file("/////")

    def test_save__main(self):
        self.project.save(aeidon.documents.MAIN)

    def test_save__translation(self):
        self.project.save(aeidon.documents.TRAN)

    def test_save__value_error(self):
        self.raises(ValueError, self.project.save, None)

    def test_save_main(self):
        for format in aeidon.formats:
            self.project.clear_texts((0,), aeidon.documents.MAIN)
            path = self.project.main_file.path
            sfile = aeidon.files.new(format, path, "ascii")
            self.project.save_main(sfile, False)
            assert self.project.main_changed == 1
            sfile = aeidon.files.new(format, path, "ascii")
            self.project.save_main(sfile, True)
            assert self.project.main_changed == 0

    def test_save_main__io_error(self):
        sfile = aeidon.files.new(aeidon.formats.SUBRIP, "/////", "ascii")
        self.raises(IOError, self.project.save_main, sfile)

    def test_save_main__unicode_error(self):
        path = self.project.main_file.path
        sfile = aeidon.files.new(aeidon.formats.SUBRIP, path, "undefined")
        self.raises(UnicodeError, self.project.save_main, sfile)

    def test_save_translation(self):
        for format in aeidon.formats:
            self.project.clear_texts((0,), aeidon.documents.TRAN)
            path = self.project.tran_file.path
            sfile = aeidon.files.new(format, path, "ascii")
            self.project.save_translation(sfile, False)
            assert self.project.tran_changed == 1
            self.project.save_translation(sfile, True)
            assert self.project.tran_changed == 0
